unit UnitFrmConfig;
{
    Purpose:
        This form configures the hotkey as well as receives messages
        when the hotkey is pressed.

        This form also reads the current configuration and passes those
        option onto the responsible form (mainly FrmMainPopup)

    Updates:
        New "SHIFT+INSERT" pasting options
        
        Moved the popup "show" options to their own panel
        New "Show Options" subitem under Popup
        New user definable function key for copy & permanent
        popup keystrokes.


        New Clipboard option - "Disable clipboard Monitoring when Scroll Lock is on"

        New look
        New way to do hotkeys

        Used ini.ReadInteger where appropriate
        Using UnitClipQueue
}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, ImgList;

type
  TFrmConfig = class(TForm)
    btnSave: TButton;
    btnCancel: TButton;
    tvOptions: TTreeView;
    pClipboard: TPanel;
    pClipboard_TextItems: TPanel;
    cbCopyIcons: TCheckBox;
    cbDisable: TCheckBox;
    Label2: TLabel;
    txtQueueSize: TEdit;
    udQueueSize: TUpDown;
    Label1: TLabel;
    txtOldHistorySize: TEdit;
    udRemovedSize: TUpDown;
    cbMonitorFilenames: TCheckBox;
    cbRemember: TCheckBox;
    cbMoveDuplicate: TCheckBox;
    pClipboard_NonText: TPanel;
    cbOtherItems: TCheckBox;
    Label7: TLabel;
    Label8: TLabel;
    txtOtherSize: TEdit;
    txtOtherSizeLimitKB: TEdit;
    udOtherSize: TUpDown;
    Label9: TLabel;
    pPopup: TPanel;
    udCharacters: TUpDown;
    txtCharacters: TEdit;
    Label6: TLabel;
    cbUseKeyboard: TCheckBox;
    cbMoveFirst: TCheckBox;
    pHotkey: TPanel;
    Shape1: TShape;
    lblKeyname: TLabel;
    Label5: TLabel;
    bEditPopupHotkey: TButton;
    pPasting: TPanel;
    rbTyping: TRadioButton;
    rbPaste: TRadioButton;
    rbClipboardOnly: TRadioButton;
    Label3: TLabel;
    ImageList1: TImageList;
    Panel1: TPanel;
    Label4: TLabel;
    rbOtherClipboardOnly: TRadioButton;
    rbOtherPaste: TRadioButton;
    cbDisableWhenScrollLock: TCheckBox;
    cbDisableSystemTrayIconNotification: TCheckBox;
    pPopupShow: TPanel;
    cbPermanentItemsSubmenu: TCheckBox;
    cbShowPermanentItemGroups: TCheckBox;
    btnEditPermanentItems: TButton;
    cbUsePermanentItems: TCheckBox;
    cbPasteAll: TCheckBox;
    cbShowLast: TCheckBox;
    cbShowCurrent: TCheckBox;
    cbShowEditHistory: TCheckBox;
    Label10: TLabel;
    Label11: TLabel;
    cboCopyClickKey: TComboBox;
    Label13: TLabel;
    Label14: TLabel;
    cboPermanentClickKey: TComboBox;
    rbPasteSI: TRadioButton;
    cboMimicClickKey: TComboBox;
    Label15: TLabel;
    Panel2: TPanel;
    rbCtrlClickRun: TRadioButton;
    rbCtrlClickMenu: TRadioButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cbKeyChange(Sender: TObject);
    procedure cbUseKeyboardClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure txtQueueSizeKeyPress(Sender: TObject; var Key: Char);
    procedure udQueueSizeClick(Sender: TObject; Button: TUDBtnType);
    procedure cbDisableClick(Sender: TObject);
    procedure cbUsePermanentItemsClick(Sender: TObject);
    procedure btnEditPermanentItemsClick(Sender: TObject);
    procedure rbTypingClick(Sender: TObject);
    procedure rbPasteClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure cbMonitorFilenamesClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cbPasteAllClick(Sender: TObject);
    procedure rbClipboardOnlyClick(Sender: TObject);
    procedure cbMoveFirstClick(Sender: TObject);
    procedure cbShowCurrentClick(Sender: TObject);
    procedure udCharactersClick(Sender: TObject; Button: TUDBtnType);
    procedure cbRememberClick(Sender: TObject);
    procedure cbCopyIconsClick(Sender: TObject);
    procedure cbMoveDuplicateClick(Sender: TObject);
    procedure cbShowLastClick(Sender: TObject);
    procedure txtOldHistorySizeKeyPress(Sender: TObject; var Key: Char);
    procedure udRemovedSizeClick(Sender: TObject; Button: TUDBtnType);
    procedure cbOtherItemsClick(Sender: TObject);
    procedure txtOtherSizeKeyPress(Sender: TObject; var Key: Char);
    procedure txtOtherSizeLimitKBChange(Sender: TObject);
    procedure udOtherSizeClick(Sender: TObject; Button: TUDBtnType);
    procedure cbShowPermanentItemGroupsClick(Sender: TObject);
    procedure tvOptionsClick(Sender: TObject);
    procedure tvOptionsCollapsing(Sender: TObject; Node: TTreeNode;
      var AllowCollapse: Boolean);
    procedure bEditPopupHotkeyClick(Sender: TObject);
    procedure cbDisableWhenScrollLockClick(Sender: TObject);
    procedure cbShowEditHistoryClick(Sender: TObject);
    procedure cbPermanentItemsSubmenuClick(Sender: TObject);
    procedure cbDisableSystemTrayIconNotificationClick(Sender: TObject);
    procedure cboCopyClickKeyClick(Sender: TObject);
    procedure cboPermanentClickKeyClick(Sender: TObject);
    procedure rbPasteSIClick(Sender: TObject);
    procedure cboMimicClickKeyClick(Sender: TObject);
    procedure rbCtrlClickRunClick(Sender: TObject);
    procedure rbCtrlClickMenuClick(Sender: TObject);
    procedure tvOptionsChange(Sender: TObject; Node: TTreeNode);




  private
    { Private declarations }
    HotKeyID: integer;

    LastShownPanel : TPanel;

    Hotkey_Ctrl : boolean;
    Hotkey_Alt : boolean;
    Hotkey_Winkey : boolean;
    Hotkey_Shift : boolean;
    Hotkey_Key :  word;

    procedure HotkeySave;
    procedure HotkeyLoad;
    function GetINIFileName: string;
    function HotKeyAssign(
            Ctrl: boolean; Alt: boolean; Shift: boolean; WinKey: boolean;
            Keystroke: word; FunctionKey : boolean): boolean;
    function HotKeyRelease(): boolean;
    function GetHumanReadableHotkey: string;

    function SetHotKey : boolean;

    procedure ShowNewPanel;
  public
    function GetHotKeyID: integer;
    procedure SaveLastPermPath;
    { Public declarations }
  end;

var
  FrmConfig: TFrmConfig;

implementation

uses
    UnitFrmPermanent,
    INIFiles,
    UnitFrmClipboardManager,
    UnitFrmMainPopup, UnitClipQueue, UnitOtherQueue, UnitFrmHotKey,
  UnitFrmSysTrayMenu, UnitFrmPermanentNew;




{$R *.dfm}




{
--========================
-- // Hook/ Release    //
--========================

}
procedure TFrmConfig.FormCreate(Sender: TObject);
var     tn : TTreeNode;
begin
    self.HotkeyLoad();

    //
    //Hook onto keystoke
    //
    HotKeyID := GlobalAddAtom('ArsClip.exe.MyHotKey');

    {
    if (rbKey.Checked) then begin
        w := ord(txtKey.text[1]);
    end else begin
        w := Hotkey_Key;
    end;
    }
    if not self.HotKeyAssign(Hotkey_Ctrl, Hotkey_Alt, Hotkey_Shift,
        Hotkey_Winkey, Hotkey_Key, false) then begin
        ShowMessage('ERROR: Unable to assign a hotkey. Please change the key.');
    end;



    tn := tvOptions.Items.GetFirstNode;
    while tn <> nil do begin
        if tn.HasChildren then
            tn.expand(true);
        tn := tn.getNextSibling;
    end;

    tvOptions.Selected := tvOptions.Items.GetFirstNode;
    tvOptions.HideSelection := false;
    tvOptions.OnClick(self);
end;


procedure TFrmConfig.FormDestroy(Sender: TObject);
begin
    if not (self.HotKeyRelease()) then begin
        ShowMessage('ERROR: Unable to release hotkey.');
    end;
    //self.HotkeySave;
end;

procedure TFrmConfig.FormShow(Sender: TObject);
begin
    Windows.SetForegroundWindow(self.Handle);
end;





function TFrmConfig.GetHotKeyID: integer;
begin
    result := HotKeyID;
end;


function TFrmConfig.GetHumanReadableHotkey : string;
var b : boolean;
begin
    result := FrmHotkey.GetHumanReadable(Hotkey_alt, Hotkey_shift, Hotkey_ctrl, Hotkey_winkey, Hotkey_key, b);
end;










{
--============================
-- // Load/Save hotkey info //
--============================

}
const ArsClip = 'ArsClip';

procedure TFrmConfig.SaveLastPermPath;
var ini: TInifile;
begin
    ini := TInifile.Create( self.GetINIFileName );

    ini.WriteString(ArsClip, 'LastPermanentPath', frmPermanent.GetPermanentPath);
    ini.Free;
end;

function TFrmConfig.GetINIFileName: string;
begin
    Result := ExtractFilePath ( application.ExeName ) + 'arsclip.ini';
end;

procedure TFrmConfig.HotkeySave;
var ini: TInifile;
begin
    ini := TInifile.Create( self.GetINIFileName );

    {
    ini.WriteString(ArsClip, 'Keystroke', txtKey.Text);
    ini.WriteInteger(ArsClip, 'Modifier', cbMod.ItemIndex );
    ini.WriteInteger(ArsClip, 'FKeystroke', cbKey.ItemIndex );
    }

    ini.WriteInteger(ArsClip,'Hotkey_Key', Hotkey_Key);
    ini.WriteBool(ArsClip,'Hotkey_Alt', Hotkey_Alt);
    ini.WriteBool(ArsClip,'Hotkey_Shift', Hotkey_Shift);
    ini.WriteBool(ArsClip,'Hotkey_Ctrl', Hotkey_Ctrl);
    ini.WriteBool(ArsClip,'Hotkey_Winkey', Hotkey_Winkey);


    //'Popup' panel

    ini.WriteBool(Arsclip, 'MoveSelectedToFirst', cbMoveFirst.Checked );

    if (cbUseKeyboard.Checked) then begin
        ini.WriteString(ArsClip, 'PopupUseKeyboard', 'Y');
    end else begin
        ini.WriteString(ArsClip, 'PopupUseKeyboard', 'N');
    end;
    ini.WriteInteger(ArsClip, 'CharactersToShow', udCharacters.position);
    ini.WriteInteger(ArsClip, 'CopyClickKey', cboCopyClickKey.ItemIndex  );
    ini.WriteInteger(ArsClip, 'PermanentClickKey', cboPermanentClickKey.ItemIndex );
    ini.WriteInteger(ArsClip, 'MimicClickKey', cboMimicClickKey.ItemIndex );
    ini.WriteBool(ArsClip, rbCtrlClickRun.Name, rbCtrlClickRun.Checked );
    ini.WriteBool(ArsClip, rbCtrlClickMenu.Name, rbCtrlClickMenu.Checked );
    
    //

    ini.WriteInteger(ArsClip, 'QueueSize', udQueueSize.position);
    ini.WriteInteger(ArsClip, 'RemovedItemsSize', udRemovedSize.Position);

    if (cbDisable.Checked) then begin
        ini.WriteString(ArsClip, 'DisableMonitoring', 'Y');
    end else begin
        ini.WriteString(ArsClip, 'DisableMonitoring', 'N');
    end;
    ini.WriteBool(ArsClip, 'DisableWhenScrollLock', cbDisableWhenScrollLock.checked);
    ini.WriteBool(ArsClip, 'CopyIcons', cbCopyIcons.checked);
    ini.WriteBool(ArsClip, 'DisableSystemTrayIconNotification', cbDisableSystemTrayIconNotification.Checked   );


    if (cbUsePermanentItems.Checked) then begin
        ini.WriteString(ArsClip, 'UsePermanentItems', 'Y');
    end else begin
        ini.WriteString(ArsClip, 'UsePermanentItems', 'N');
    end;
    ini.WriteString(ArsClip, 'LastPermanentPath', frmPermanent.GetPermanentPath);

    ini.WriteBool(ArsClip,'ShowPermanentItemGroups', cbShowPermanentItemGroups.checked );
    ini.WriteBool(ArsClip, 'ShowPermanentItemSubmenu', cbPermanentItemsSubmenu.Checked );

    if (rbTyping.Checked) then begin
        ini.WriteString(ArsClip,'PasteMethod', 'MimicTyping');
    end else if (rbPaste.checked) then begin
        ini.WriteString(ArsClip, 'PasteMethod', 'CTRL+V');
    end else if (rbPasteSI.Checked) then begin
        ini.WriteString(ArsClip, 'PasteMethod', 'SHIFT+INSERT');
    end else begin
        ini.WriteString(ArsClip, 'PasteMethod', 'ClipboardOnly');
    end;

    ini.WriteBool(ArsClip, 'MonitorFilenames', cbMonitorFilenames.checked);
    ini.WriteBool(ArsClip, 'ShowPasteAll', cbPasteAll.checked );

    ini.WriteBool(ArsClip, 'ShowCurrent', cbShowCurrent.checked );

    ini.WriteBool(ArsClip, 'RememberHistory', cbRemember.checked);

    ini.WriteBool(ArsClip, 'MoveDuplicate', cbMoveDuplicate.checked );
    ini.WriteBool(ArsClip, 'ShowLast', cbShowLast.checked);
    ini.WriteBool(ArsClip, 'ShowEditHistory', cbShowEditHistory.Checked);

    ini.WriteBool(ArsClip,'EnableOtherItems', cbOtherItems.Checked);
    ini.WriteInteger(ArsClip, 'OtherSize', udOtherSize.Position);
    ini.WriteInteger(ArsClip,'OtherSizeLimitKB', StrToInt(txtOtherSizeLimitKB.text));

    if (rbOtherPaste.Checked) then begin
        ini.WriteString(ArsClip, 'OtherPasteMethod', 'CTRL+V' );
    end else begin
        ini.WriteString(ArsClip, 'OtherPasteMethod',  'ClipboardOnly');
    end;


    //lblKeyname.Caption := self.GetHumanReadableHotkey;
    ini.Free;
end;

procedure TFrmConfig.HotkeyLoad;
var ini: TInifile;
begin
    //
    // read last assigned hotkey
    // CTRL+SHIFT+Z is default
    //
    // fake the user selecting the Modifier and the Normal Key

    ini := TInifile.Create( self.GetINIFileName );

    Hotkey_Key := ini.ReadInteger(ArsClip,'Hotkey_Key', Ord('Z'));
    Hotkey_Alt := ini.ReadBool(ArsClip,'Hotkey_Alt', false);
    Hotkey_Shift := ini.ReadBool(ArsClip,'Hotkey_Shift', true);
    Hotkey_Ctrl := ini.ReadBool(ArsClip,'Hotkey_Ctrl', true);
    Hotkey_Winkey := ini.ReadBool(ArsClip,'Hotkey_Winkey', false);

    //----------------
    // 'Popup' panel
    //----------------
    cbMoveFirst.Checked := ini.ReadBool(ArsClip, 'MoveSelectedToFirst', false);
    cbUseKeyboard.Checked := (ini.ReadString(ArsClip,'PopupUseKeyboard', 'Y') = 'Y');
    udCharacters.position := ini.ReadInteger(ArsClip, 'CharactersToShow', 30);


    cboCopyClickKey.ItemIndex :=     ini.ReadInteger(ArsClip, 'CopyClickKey', 0 );
    cboPermanentClickKey.ItemIndex :=    ini.ReadInteger(ArsClip, 'PermanentClickKey', 1);
    cboMimicClickKey.ItemIndex := ini.ReadInteger(ArsClip, 'MimicClickKey', 2 );
    rbCtrlClickRun.Checked := ini.ReadBool(ArsClip, rbCtrlClickRun.Name, false);
    rbCtrlClickMenu.Checked := ini.ReadBool(ArsClip, rbCtrlClickMenu.name, true );
    
    self.cboCopyClickKeyClick(self);
    self.cboPermanentClickKeyClick(self);
    self.cboMimicClickKeyClick(self);
    // setting the ItemIndex doesn't auto set of any event




    //'s := ini.ReadString(ArsClip, 'QueueSize', '15');
    //udqueuesize.Position  := StrToInt(s);
    udqueuesize.Position  := ini.ReadInteger(ArsClip, 'QueueSize', 15);
    udRemovedSize.Position := ini.ReadInteger(ArsClip, 'RemovedItemsSize', 50);

    ClipQueue.SetQueueSize(udQueueSize.position);
    RemovedQueue.SetQueueSize(udRemovedSize.position);

    cbDisable.Checked := (ini.ReadString(ArsClip,'DisableMonitoring','N') = 'Y');
    cbCopyIcons.Checked := ini.ReadBool(ArsClip,'CopyIcons', true);
    cbDisableSystemTrayIconNotification.Checked := ini.ReadBool(ArsClip,'DisableSystemTrayIconNotification', false);
    cbDisableWhenScrollLock.Checked := ini.ReadBool(ArsClip, 'DisableWhenScrollLock', false);
    cbUsePermanentItems.Checked := (ini.ReadString(ArsClip,'UsePermanentItems', 'N') = 'Y');
    cbShowPermanentItemGroups.checked := ini.ReadBool(ArsClip,'ShowPermanentItemGroups', true );
    cbPermanentItemsSubmenu.checked := ini.Readbool(ArsClip,'ShowPermanentItemSubmenu', false);

    cbUseKeyboard.OnClick(self);
    cbDisable.OnClick(self);
    cbUsePermanentItems.OnClick(self);

    frmPermanent.SetPermanentPath(ini.ReadString(ArsClip, 'LastPermanentPath', UnitFrmPermanent.DEFAULT_FOLDER));

    rbTyping.Checked := ini.ReadString(ArsClip, 'PasteMethod', 'CTRL+V') = 'MimicTyping';
    rbPaste.Checked := ini.ReadString(ArsClip, 'PasteMethod', 'CTRL+V') = 'CTRL+V';
    rbClipboardOnly.checked := ini.ReadString(ArsClip, 'PasteMethod', 'CTRL+V') = 'ClipboardOnly';
    rbPasteSI.Checked := ini.ReadString(ArsClip, 'PasteMethod', 'SHIFT+INSERT') = 'SHIFT+INSERT';

    cbMonitorFilenames.Checked :=  ini.ReadBool(ArsClip, 'MonitorFilenames', false);
    cbPasteAll.Checked := ini.ReadBool(ArsClip, 'ShowPasteAll', true);

    cbShowCurrent.Checked := ini.ReadBool(ArsClip, 'ShowCurrent', false );

    frmMainPopup.SetVisibleCharacterCount(udCharacters.position);
    cbRemember.Checked := ini.ReadBool(ArsClip, 'RememberHistory', true);
    cbMoveDuplicate.checked := ini.ReadBool(ArsClip, 'MoveDuplicate', true);
    cbShowLast.Checked := ini.ReadBool(ArsClip,'ShowLast', true);
    cbShowEditHistory.checked := ini.ReadBool(ArsClip,'ShowEditHistory', false);

    cbOtherItems.Checked := ini.ReadBool(ArsClip,'EnableOtherItems', false);
    udOtherSize.Position := ini.ReadInteger(ArsClip, 'OtherSize', 5);
    txtOtherSizeLimitKB.text := IntToStr(ini.ReadInteger(ArsClip,'OtherSizeLimitKB', 500));

    rbOtherClipboardOnly.Checked := ini.ReadString(ArsClip, 'OtherPasteMethod',  'ClipboardOnly') = 'ClipboardOnly';
    rbOtherPaste.Checked := ini.ReadString(ArsClip, 'OtherPasteMethod',  'ClipboardOnly') = 'CTRL+V';


    ini.free;

    //
    // these are here because because there is no "onchange"
    // triggered when the control's state is changed
    //
    if (rbOtherPaste.Checked) then begin
        frmMainPopup.SetOtherPasteMethodPasting;
    end else begin
        FrmMainPopup.SetOtherPasteMethodClipboardOnly;
    end;

    if rbTyping.checked then
        frmMainPopup.SetPasteMethodMimicKeyboard;
    if rbClipboardOnly.checked then
        frmMainPopup.SetPasteMethodClipboardOnly;
    if rbPaste.Checked then
        frmMainPopup.SetPasteMethodPasting;
    if rbPasteSI.Checked then
        FrmMainPopup.SetPasteMethodPastingSI;
         
    lblKeyname.Caption := self.GetHumanReadableHotkey;
end;


{
--========================
-- // Release/Assign    //
--========================

Returns true if successful.

}

function TFrmConfig.HotKeyRelease(): boolean;
begin
    result := UnRegisterHotKey(frmMainPopup.handle, HotKeyID)
end;

function TFrmConfig.HotKeyAssign(
    Ctrl: boolean; Alt: boolean; Shift: boolean; Winkey : boolean;
    Keystroke: word; FunctionKey : boolean): boolean;
var modifier: word;
begin
    modifier := 0;
    if ctrl then modifier := modifier or MOD_CONTROL;
    if alt then  modifier := modifier or MOD_ALT;
    if shift then modifier := modifier or MOD_SHIFT;
    if winkey then modifier := modifier or MOD_WIN;
    result := false;
    if (HotKeyID = 0) then begin
        exit;
    end;


    // this alteration allows for using , . ` in hotkeys
    //if (not functionkey) then begin
    //    keystroke := VkKeyScan(char(Keystroke)) and $ff;
    //end;
    if not ( RegisterHotKey(frmMainPopup.handle, HotKeyID, modifier, Keystroke)) then begin
        exit;
    end;

    result := true;
end;









{
--========================
-- // Control Events    //
--========================

}
function TFrmConfig.SetHotKey : boolean;
begin
    result := false;

    //
    // if the user tries to assign to a hotkey that is already in use,
    // the hotkeyrelease failing is OK
    //
    self.HotKeyRelease();


    if not (self.HotKeyAssign(Hotkey_Ctrl, Hotkey_Alt, Hotkey_Shift, Hotkey_Winkey,
        Hotkey_key, false)) then begin
        ShowMessage('ERROR: Couldn''t assign hotkey. It may already be assigned by another program.');
    end else begin
        result := true;
    end;
end;



procedure TFrmConfig.cbKeyChange(Sender: TObject);
begin
end;


//============================
// Clipboard Options
//============================

procedure TFrmConfig.cbUseKeyboardClick(Sender: TObject);
begin
    frmMainPopup.setpopupusekeyboard(cbUseKeyboard.Checked);
end;

{
Make the user use the up/down to change the queue size
}
procedure TFrmConfig.txtQueueSizeKeyPress(Sender: TObject; var Key: Char);
begin
    // ignore keystrokes
    key := chr(0);
end;

procedure TFrmConfig.txtOldHistorySizeKeyPress(Sender: TObject;
  var Key: Char);
begin
    // ignore keystrokes
    Key := Chr(0);
end;

procedure TFrmConfig.txtOtherSizeKeyPress(Sender: TObject; var Key: Char);
begin
    key := chr(0);
end;


procedure TFrmConfig.cbOtherItemsClick(Sender: TObject);
begin
    udOtherSize.Enabled := cbOtherItems.Checked;
    txtOtherSizeLimitKB.Enabled := cbOtherItems.Checked;
    frmClipboardManager.SetOtherItemsMonitoring(cbOtherItems.Checked);

end;

procedure TFrmConfig.txtOtherSizeLimitKBChange(Sender: TObject);
begin
    OtherQueue.SetClipSizeLimit( StrToInt(txtOtherSizeLimitKB.text) * 1024);
end;

procedure TFrmConfig.udOtherSizeClick(Sender: TObject; Button: TUDBtnType);
begin
    OtherQueue.SetQueueSize(udOtherSize.position);
end;


procedure TFrmConfig.udQueueSizeClick(Sender: TObject; Button: TUDBtnType);
begin
    ClipQueue.SetQueueSize(udQueueSize.position);
end;
procedure TFrmConfig.udRemovedSizeClick(Sender: TObject; Button: TUDBtnType);
begin
    RemovedQueue.SetQueueSize(udRemovedSize.position);
end;


procedure TFrmConfig.cbDisableClick(Sender: TObject);
begin
    frmClipboardManager.SetMonitoring(cbdisable.Checked);
end;

procedure TFrmConfig.cbMonitorFilenamesClick(Sender: TObject);
begin
    frmClipboardManager.SetFilenameMonitoring(cbMonitorFilenames.checked);
end;
procedure TFrmConfig.cbRememberClick(Sender: TObject);
begin
    frmMainPopup.SetRememberHistory(cbRemember.checked);
    // this loads only the first time
    if (cbRemember.Checked) then begin
        frmClipboardManager.LoadHistory;
    end;
end;

procedure TFrmConfig.cbCopyIconsClick(Sender: TObject);
begin
    frmClipboardManager.SetCopyIcon(cbCopyIcons.Checked);
end;

procedure TFrmConfig.cbMoveDuplicateClick(Sender: TObject);
begin
    ClipQueue.SetMoveDuplicateTop(cbMoveDuplicate.checked);
end;




procedure TFrmConfig.cbUsePermanentItemsClick(Sender: TObject);
begin
    frmMainPopup.SetPermanentItems(cbUsePermanentItems.Checked);
end;


procedure TFrmConfig.cbShowEditHistoryClick(Sender: TObject);
begin
    FrmMainPopup.SetShowEditHistory(cbShowEditHistory.checked);
end;

procedure TFrmConfig.cbShowPermanentItemGroupsClick(Sender: TObject);
begin
    if (cbShowPermanentItemGroups.Checked) then begin
        cbPermanentItemsSubmenu.Checked := not cbShowPermanentItemGroups.Checked;
    end;
    FrmMainPopup.SetShowPermeanentItemGroups(cbShowPermanentItemGroups.checked);
end;

procedure TFrmConfig.cbPermanentItemsSubmenuClick(Sender: TObject);
begin
    if (cbPermanentItemsSubmenu.Checked) then begin
        cbShowPermanentItemGroups.Checked := not cbPermanentItemsSubmenu.Checked;
    end;
    FrmMainPopup.SetShowPermanentItemSubmenu(cbPermanentItemsSubmenu.Checked);
end;



procedure TFrmConfig.btnEditPermanentItemsClick(Sender: TObject);
begin
    frmPermanent.ShowModal;
end;

procedure TFrmConfig.cbPasteAllClick(Sender: TObject);
begin
    frmMainPopup.SetShowPasteAll(cbPasteAll.checked);
end;


//============================
// Pasting Options
//============================
procedure TFrmConfig.rbTypingClick(Sender: TObject);
begin
    frmMainPopup.SetPasteMethodMimicKeyboard();
end;
procedure TFrmConfig.rbPasteClick(Sender: TObject);
begin
    frmMainPopup.SetPasteMethodPasting();
end;
procedure TFrmConfig.rbPasteSIClick(Sender: TObject);
begin
    frmMainPopup.SetPasteMethodPastingSI; 
end;
procedure TFrmConfig.rbClipboardOnlyClick(Sender: TObject);
begin
    frmMainPopup.SetPasteMethodClipboardOnly();
end;






procedure TFrmConfig.cbMoveFirstClick(Sender: TObject);
begin
    frmMainPopup.SetMoveSelectedItemToTop(cbmovefirst.checked);
end;
procedure TFrmConfig.cbShowCurrentClick(Sender: TObject);
begin
    frmMainPopup.SetShowCurrentItem(cbSHowCurrent.checked);
end;

procedure TFrmConfig.cbShowLastClick(Sender: TObject);
begin
    frmMainPopup.SetDoShowLast(cbShowLast.checked);
end;

procedure TFrmConfig.udCharactersClick(Sender: TObject;
  Button: TUDBtnType);
begin
    frmMainPopup.SetVisibleCharacterCount(udCharacters.Position);
end;



//
// close/save/cancel
// TreeView clicks
//
procedure TFrmConfig.btnSaveClick(Sender: TObject);
begin
    self.HotkeySave;
    if (self.SetHotKey) then begin
        self.close;
    end;
end;

procedure TFrmConfig.btnCancelClick(Sender: TObject);
begin
    self.Close;
end;

procedure TFrmConfig.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    self.HotkeyLoad;
end;





procedure TFrmConfig.ShowNewPanel;
begin
    if tvOptions.Selected <> nil then begin
        if LastShownPanel <> nil then
            LastShownPanel.Visible := false;

        case (tvOptions.Selected.AbsoluteIndex) of
        0:  begin
                pHotkey.Visible := true;
                LastShownPanel := pHotkey;
            end;
        1:  begin
                pClipboard.Visible := true;
                LastShownPanel := pClipboard;
            end;
        2:  begin
                pClipboard_TextItems.Visible := true;
                LastShownPanel := pClipboard_TextItems;
            end;
        3:  begin
                pClipboard_NonText.Visible := true;
                LastShownPanel := pClipboard_NonText;
            end;
        4:  begin
                pPasting.Visible := true;
                LastShownPanel := pPasting;
            end;
        5:  begin
                pPopup.Visible := true;
                LastShownPanel := pPopup;
            end;
        6:  begin
                pPopupShow.Visible := true;
                LastShownPanel := pPopupShow;
            end;
        else
            if LastShownPanel <> nil then
                LastShownPanel.Visible := true;
        end;
    end;
end;

procedure TFrmConfig.tvOptionsClick(Sender: TObject);
begin
    self.ShowNewPanel;
end;

procedure TFrmConfig.tvOptionsCollapsing(Sender: TObject; Node: TTreeNode;
  var AllowCollapse: Boolean);
begin
    AllowCollapse := false;
end;

procedure TFrmConfig.bEditPopupHotkeyClick(Sender: TObject);
var w : word;
    s : string;
    ha, hs, hc, hw : boolean;
begin
    if FrmHotkey.GetHotkey( ha, hs ,hc , hw, w, s) then begin
       lblKeyname.Caption := s;
       Hotkey_Alt    := ha;
       Hotkey_Shift  := hs;
       Hotkey_Ctrl   := hc;
       Hotkey_Winkey := hw;
       Hotkey_Key    := w;
       lblKeyname.caption := s;
    end;
end;

procedure TFrmConfig.cbDisableWhenScrollLockClick(Sender: TObject);
begin
    frmClipboardManager.SetDisableWhenScrollLock(cbDisableWhenScrollLock.checked);
end;


procedure TFrmConfig.cbDisableSystemTrayIconNotificationClick(
  Sender: TObject);
begin
    FrmSysTrayMenu.SetDisableIconChanges(cbDisableSystemTrayIconNotification.checked);
end;

procedure TFrmConfig.cboCopyClickKeyClick(Sender: TObject);
begin
     FrmMainPopup.SetCopyKey(self.cboCopyClickKey.ItemIndex);
end;

procedure TFrmConfig.cboPermanentClickKeyClick(Sender: TObject);
begin
   FrmMainPopup.SetPermanentKey(self.cboPermanentClickKey.ItemIndex);
end;

procedure TFrmConfig.cboMimicClickKeyClick(Sender: TObject);
begin
    FrmMainPopup.SetMimicKey(self.cboMimicClickKey.ItemIndex);
end;

procedure TFrmConfig.rbCtrlClickRunClick(Sender: TObject);
begin
    FrmMainPopup.SetCtrlClickOpen(rbCtrlClickRun.checked); 
end;

procedure TFrmConfig.rbCtrlClickMenuClick(Sender: TObject);
begin
    FrmMainPopup.SetCtrlClickMenu(rbCtrlClickMenu.checked); 
end;

procedure TFrmConfig.tvOptionsChange(Sender: TObject; Node: TTreeNode);
begin
    self.ShowNewPanel;
end;

end.






