unit UnitFrmMainPopup;
{
    This unit displays the main popup menu when a hotkey is pressed

    UnitFrmConfig is responsible for setting/releasing the hotkey -
    I had a good reason why, but I don't remeber it. I think there
    was a circular reference where each form required the other during
    creation.


    NOTE:
    Ignore the clipboard from Hotkey until Popup Dismissal
    Ignore the clipboard from paste start until paste end

    Inform the FrmClipboardManager any time we paste an item.
    (in routines SendText and OtherItemClickEvent)

Update:
    New secondary popup menu for text items (alternative to
    function key + clicking on items)

    Updated the focus rules for asking the user about "no-paste" windows
    in the hotkey message method.

    --

    New SHIFT+INSERT pasting method

    Shift + Click to preview a picture (non-text item)
    F1 + Click to copy to clipboard only a non-text item

    Non-text items shown in Newest-To-Oldest order

    Flush non-text items command in Full Mode
    Flush text items command in Full Mode
    
    --

    Show configured (either on popup or in submenu) for
    permanent item groups in full mode - default to on popup when not
    specified

    New F1+Click to copy to clipboard only
    New F2+Click to make into a new Permanent Item
    Cleaned up special Key+Click code

    
    New 'Full Mode/Configured Mode' toggle.

    Code cleanup - made SendCTRL_V procedure.

    Allow user to specify "NoPaste" for programs that don't
    have a target control.

    New option for quick-change of Permanent Item groups

    CTRL+Clicks on Menu Items, Last, Current, Permanent Items will open
    files, urls, folders, etc.

    Using UnitClipQueue
    New handling for tabs in the display of clip items
    Focus is now returned when ESC is pressed

}
interface
uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Clipbrd, Menus, StrUtils, ShellAPI, ExtCtrls, Buttons,
  IniFiles {for hashed};


  type
  TFrmMainPopup = class(TForm)
    Memo1: TMemo;
    xlbQueue: TListBox;
    btnHide: TButton;
    imgA: TImage;
    imgRightArrow: TImage;
    imgIcon: TImage;
    sbIcon: TSpeedButton;
    imgFull: TImage;
    pmItemOptions: TPopupMenu;
    Run1: TMenuItem;
    MakePermanentItem1: TMenuItem;
    PasteusingMimicTyping1: TMenuItem;
    Placeonclipboard1: TMenuItem;
    Preview1: TMenuItem;
    back1: TMenuItem;
    Cancel1: TMenuItem;
    N2: TMenuItem;
    PasteusingSHIFTINS1: TMenuItem;
    N1: TMenuItem;
    PasteusingCTRLV1: TMenuItem;
    imgPasteAll: TImage;
    imgPaste: TImage;
    imgEdit: TImage;
    imgFlush: TImage;
    imgConfMode: TImage;
    imgPermCasc: TImage;
    imgRemoved: TImage;

    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnHideClick(Sender: TObject);
    procedure Run1Click(Sender: TObject);
    procedure MakePermanentItem1Click(Sender: TObject);
    procedure PasteusingMimicTyping1Click(Sender: TObject);
    procedure Placeonclipboard1Click(Sender: TObject);
    procedure Preview1Click(Sender: TObject);
    procedure back1Click(Sender: TObject);
    procedure PasteusingSHIFTINS1Click(Sender: TObject);
    procedure PasteusingCTRLV1Click(Sender: TObject);
  private
    { Private declarations }

    HotKeyForeWindow: cardinal;         {target window for text pasting}
    HotKeyTarget: cardinal;             {target control with keyboard focus}

    Popup: TPopupMenu;                  {text pasting popup}

    LastStringSelected : string;         {last selected queue item from Popup}
    MenuItemText : string;

    ThreadOurs : cardinal;               {Used soley by ThreadAttach/Detach}
    ThreadTarget : cardinal;
    ThreadAttached : boolean;

    PopupUseKB : boolean;
    VisibleCharCount : longint;
    UsePermanentItems : boolean;
    DoShowEditHistory : boolean;
    DoShowPasteAll : boolean;
    DoMoveSelectedToTop : boolean;
    DoShowCurrentItem : boolean;
    DoShowPermanentItemGroups : boolean;
    DoShowPermanentItemSubmenu : boolean;
    DoRememberHistory : boolean;
    DoShowLast : boolean;

    {keystrokes monitored when an item is clicked}
    CopyKey : integer;
    PermanentKey : integer;
    MimicKey : integer;

    {"pasting methods"}
    {text items}
    UseKeyboardMimic : boolean;
    UsePasting : boolean;
    UseClipboardOnly : boolean;
    UsePastingSI : boolean;
    
    {non-text items}
    UseOtherClipboardOnly : boolean;
    UseOtherPasting : boolean;

    UseCtrlClickOpen, UseCtrlClickMenu : boolean;

    {paste overrides}
    ClipboardOnlyWindow : boolean;  // temp toggle to force a "copy and no
                                    // paste" when sending the text or other format
                                    // set in WMHotKey
                                    // used in SendText and OtherMenuItemClicEvent
    MimicTypingOnce : boolean;      // temp toggle to force mimic typing for
                                    // pasting
                                    // set in WMHotKey
                                    // used in SendText
    ShiftInsertOnce : boolean;
    CtrlVOnce : boolean;



    FullMode : boolean; // user pressed Full Mode command

    TargetX, TargetY : integer;

    NoPastePrograms : THashedStringList;    // List of programs that ArsClip will use
                                            // clipboard only mode.

    procedure HandleCloseEvent;

    function ThreadAttach(TargetWindow: cardinal) : boolean;
    procedure ThreadDetach();

    procedure GetTargetControl(ParentWindow: cardinal;
        var TargetHandle: cardinal;
        var TargetX: integer;
        var TargetY: integer);


    procedure ShowPopupWithFocusRules;
    procedure ShowPopup(X,Y: integer);

    //
    // menu items selected on Popup window
    //
    procedure MenuItemClickEvent(Sender: TObject);
    procedure FlushTextItemsClickEvent(Sender: TObject);
    procedure EditHistoryItemClickEvent(Sender : TObject);
    procedure OtherItemClickEvent(Sender: TObject);
    procedure FlushOtherItemsClickEvent(Sender: TObject);
    procedure LastMenuItemClickEvent(Sender: TObject);
    procedure CancelMenuItemClickEvent(Sender: TObject);
    procedure PermanentMenutItemClickEvent(sender: TObject);
    procedure PermanentGroupItemClickEvent(sender: TObject);
    procedure PasteAllMenuItemClickEvent(sender: TObject);
    procedure PasteSelectedMenuItemClickEvent(sender: TOBject);
    procedure CurrentMenuItemClickEvent(Sender: TObject);
    procedure FullModeMenuItemClickEvent(Sender : TObject);
    //
    // util functions
    //
    procedure ShowPreviewForm;
    function KeyPressedOverride(s : string) : boolean;

    //function CTRLOpen(s : string) : boolean;
    //function SHIFTOpen(s : string) : boolean;
    procedure SendText(s : string);
    procedure SendCTRL_V;
    procedure SendSHIFT_INSERT;

    //procedure PlaceTextOnClipboard( s : string);

  protected
  public
    {public declarations}

    {configuration}
    procedure SetPopupUseKeyboard(usekb: boolean);
    procedure SetVisibleCharacterCount(count: longint);
    procedure SetPermanentItems(DoUse: boolean);
    procedure SetShowPermeanentItemGroups(enable : boolean);
    procedure SetShowPermanentItemSubmenu(value : boolean);
    procedure SetPasteMethodMimicKeyboard();
    procedure SetPasteMethodPasting();
    procedure SetPasteMethodPastingSI();
    procedure SetPasteMethodClipboardOnly();
    procedure SetOtherPasteMethodPasting();
    procedure SetOtherPasteMethodClipboardOnly();
    procedure SetShowEditHistory(value : boolean);

    procedure SetShowPasteAll(DoShow: boolean);
    procedure SetMoveSelectedItemToTop(DoMove : boolean);
    procedure SetShowCurrentItem(DoShow: boolean);
    procedure SetRememberHistory(DoRemember: boolean);
    procedure SetDoShowLast(DoShow: boolean);

    procedure SetCopyKey(key : integer);
    procedure SetPermanentKey(key : integer);
    procedure SetMimicKey(key : integer);
    procedure SetCtrlClickOpen(value : boolean);
    procedure SetCtrlClickMenu(value : boolean);
    {windows messages}

    {system tray stuff/ windows messages}
    procedure WMHotKey(var Msg : TWMHotKey); message WM_HOTKEY;
    procedure WMEndSession(var msg : TWMQueryEndSession); message WM_QUERYENDSESSION;
    procedure AppendLog(s : string);
  end;

var
  FrmMainPopup: TFrmMainPopup;
implementation

uses UnitFrmConfig, UnitFrmPermanent, UnitFrmClipboardManager,
  UnitFrmPasteSelected, UnitClipQueue, UnitOtherQueue, UnitKeyboardQuery,
  TLHelp32, UnitFrmPreview, UnitFrmPermanentNew;

CONST PASTE_ONLY_FILE : string = 'nopaste.txt';

{$R *.dfm}

{
--========================
-- // Public Interface //
--========================
}

procedure TFrmMainPopup.SetCtrlClickOpen(value : boolean);
begin
    self.UseCtrlClickOpen := value;
    self.UseCtrlClickMenu := not value;
end;
procedure TFrmMainPopup.SetCtrlClickMenu(value : boolean);
begin
    self.UseCtrlClickOpen := not value;
    self.UseCtrlClickMenu := value;
end;

procedure TFrmMainPopup.SetMimicKey(key : integer);
begin
    self.MimicKey := VK_F1 + key;
end;

procedure TFrmMainPopup.SetCopyKey(key : integer);
begin
    self.CopyKey := VK_F1 + key;
end;

procedure TFrmMainPopup.SetPermanentKey(key : integer);
begin
    self.PermanentKey := VK_F1 + key;
end;

procedure TFrmMainPopup.SetShowEditHistory(value : boolean);
begin
    self.DoShowEditHistory := value;
end;

procedure TFrmMainPopup.SetShowPermanentItemSubmenu(value : boolean);
begin
    if (value) then begin
        DoShowPermanentItemSubmenu := true;
        DoShowPermanentItemGroups := false;
    end;
end;
procedure TFrmMainPopup.SetShowPermeanentItemGroups(enable : boolean);
begin
    if (enable) then begin
        DoShowPermanentItemSubmenu := false;
        DoShowPermanentItemGroups := true;
    end;
end;
procedure TFrmMainPopup.SetOtherPasteMethodPasting();
begin
    self.UseOtherClipboardOnly := false;
    self.UseOtherPasting := true;
end;
procedure TFrmMainPopup.SetOtherPasteMethodClipboardOnly();
begin
    self.UseOtherClipboardOnly := true;
    self.UseOtherPasting := false;
end;

procedure TFrmMainPopup.SetDoShowLast(DoShow: boolean);
begin
    self.DoShowLast := DoShow;
end;
procedure TFrmMainPopup.SetRememberHistory(DoRemember: boolean);
begin
    self.DoRememberHistory := DoRemember;
end;
procedure TFrmMainPopup.SetVisibleCharacterCount(count: longint);
begin
    self.VisibleCharCount := count;
end;
procedure TFrmMainPopup.SetShowCurrentItem(DoShow: boolean);
begin
    self.DoShowCurrentItem := doshow;
end;

procedure TFrmMainPopup.SetShowPasteAll(DoShow: boolean);
begin
    self.DoShowPasteAll := DoShow;
end;
procedure TFrmMainPopup.SetPopupUseKeyboard(usekb: boolean);
begin
    self.PopupUseKB := usekb;
end;

procedure TFrmMainPopup.SetPermanentItems(DoUse: boolean);
begin
    self.UsePermanentItems := douse;
end;


procedure TFrmMainPopup.SetPasteMethodMimicKeyboard();
begin
    UsePasting := false;
    UseClipboardOnly := false;
    UseKeyboardMimic := true;
    UsePastingSI := false;
end;

procedure TFrmMainPopup.SetPasteMethodPasting();
begin
    UsePasting := true;
    UseClipboardOnly := false;
    UseKeyboardMimic := false;
    UsePastingSI := false;
end;

procedure TFrmMainPopup.SetPasteMethodPastingSI();
begin
    UsePasting := false;
    UseClipboardOnly := false;
    UseKeyboardMimic := false;
    UsePastingSI := true;
end;
procedure TFrmMainPopup.SetPasteMethodClipboardOnly();
begin
    UsePasting := false;
    UseClipboardOnly := true;
    UseKeyboardMimic := false;
    UsePastingSI := false;
end;

procedure TFrmMainPopup.SetMoveSelectedItemToTop(DoMove : boolean);
begin
    DoMoveSelectedToTop := DoMove;
end;


{
--========================
-- // Create/Destroy   //
--========================

Popup menu
NoPaste file
Init vars

Handle a form close or a logoff/shutdown
}
procedure TFrmMainPopup.FormCreate(Sender: TObject);
var f : string;
begin
    //
    // popup & menu items init
    //
    self.ShowHint := true;
    Popup := TPopupMenu.Create(self);

    ThreadAttached := false;
    UseKeyboardMimic := false;

    NoPastePrograms := THashedStringList.Create;
    f := IncludeTrailingPathDelimiter(ExtractFilePath(Application.ExeName)) +
        PASTE_ONLY_FILE;
    If FileExists(f) then begin
        NoPastePrograms.LoadFromFile(f);
    end;
end;


procedure TFrmMainPopup.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    self.HandleCloseEvent;
end;

//
// When OS is shutting down/logging off
//
procedure TFrmMainPopup.WMEndSession(var msg : TWMQueryEndSession);
begin
    self.HandleCloseEvent;
    msg.Result := 1;       // do allow close
end;


procedure TFrmMainPopup.HandleCloseEvent;
var f : string;
begin
    if self.DoRememberHistory then begin
        frmClipboardManager.SaveHistory;
    end;

    if NoPastePrograms.Count > 0 then begin
        f := IncludeTrailingPathDelimiter(ExtractFilePath(Application.ExeName)) +
          PASTE_ONLY_FILE;
        NoPastePrograms.SaveToFile(f);
    end;
    //
    // clean up resources
    //
    NoPastePrograms.Free;
    Popup.Items.Clear;
    Popup.Free;

    //
    FrmConfig.SaveLastPermPath;
end;









{
--======================
-- // HotKey Message  //
--======================

Description:
    Find the target control
    - Handle case when no target control
    - allow user to add problematic program to "No Paste" list

    Finally, pass control to  ShowPopupWithFocusRules


info collected here:
    HotKeyForeWindow
    HotKeyTarget
    TargetX
    TargetY
}


function WindowHandleToEXEName( handle : THandle) : string;
var snap : THandle;
    pe : tagPROCESSENTRY32;
    pid : THandle;
    found : boolean;
begin
    snap := TLHelp32.CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
    if (snap = Cardinal(-1)) then begin
        EXIT;
    end;

    Windows.GetWindowThreadProcessId(handle, pid);
    pe.dwSize := Sizeof(pe);
    found := TLHelp32.Process32First(snap, pe);

    while found do begin
        if (pe.th32ProcessID = pid) then begin
            result := String(pe.szExeFile);
            break;
        end;

        found := TLHelp32.Process32Next(snap, pe);
    end;
    CloseHandle(snap);
end;


procedure TFrmMainPopup.WMHotKey(var Msg: TWMHotKey);
var wp : tagWINDOWPLACEMENT;
    s : string;
    answer : longint;
    //h : THandle;
begin
    //--
    //-- Insert the clipboard info into they control the user is typing in
    //--
    if (msg.HotKey <> FrmConfig.GetHotKeyID) then begin
        exit
    end;

    //
    // ignore the clipboard as soon as the hotkey is pressed
    // stop ignoring on an error exit
    // each menu item is responsible to stop ingoring the clipboard
    frmClipboardManager.SetIgnoreClipboard(true);

    AppendLog('WM_Hotkey - received');

    // get current foreground window
    self.HotKeyForeWindow := Windows.GetForegroundWindow();
    if (self.HotKeyForeWindow = 0) then begin
        ShowMessage('Coulnd''t get foreground window');
        frmClipboardManager.SetIgnoreClipboard(false);
        exit;
    end;

    // find control with focus on foreground window
    // ingore error when pasting option is "Clipboard Only"

    self.GetTargetControl(self.HotKeyForeWindow, self.HotKeyTarget,
        TargetX, TargetY);
    ClipboardOnlyWindow := false;
    if (HotKeyTarget = 0) then begin
        Windows.GetWindowPlacement(self.HotKeyForeWindow, @wp);
        TargetX := wp.rcNormalPosition.Left;
        TargetY := wp.rcNormalPosition.Top;

        //
        // Bitch if were not in ClipboardOnly mode
        // Allow user to always use ClipboardOnly for the problematic
        // program
        // [Answer Yes = add EXE name to NoPaste file]
        //
        if (not UseClipboardOnly) then begin
            s := WindowHandleToEXEName(self.HotKeyForeWindow);
            answer := mryes;
            if NoPastePrograms.IndexOf(s) = -1 then begin
                Windows.SetForegroundWindow(self.Handle);
                answer := MessageDlg('Coudn''t find control to target in program ' + uppercase(s) + #13#10
                    + 'Would you like to use "Copy to clipboard only" for this program?',
                    mtConfirmation, [mbyes, mbno],0 );

                if (answer = mryes) then begin
                    NoPastePrograms.Add(s);
                end;
            end;

            if (answer <> mryes) then begin
                Windows.SetForegroundWindow(self.HotKeyForeWindow); 
                frmClipboardManager.SetIgnoreClipboard(false);
                EXIT;
            end;
            ClipboardOnlyWindow := True;
        end;

        Windows.SetForegroundWindow(self.Handle);

        {This wouldn't return focus when ESC was pressed.
        now using normal focus rules for this exception also.}
        //self.ShowPopup(TargetX, TargetY);
        //EXIT;
    end;

    self.FullMode := false;
    self.ShowPopupWithFocusRules;
end;


procedure TfrmMainPopup.GetTargetControl(ParentWindow: cardinal;
    var TargetHandle: cardinal;
    var TargetX: integer;
    var TargetY: integer);
var CaretPos: TPoint;
    Rect: TRect;
begin
    //
    // attempt to get target control's handle and
    // the position of the caret in the text window
    //
    TargetHandle := 0;
    if (self.ThreadAttach(ParentWindow)) then begin
        TargetHandle := Windows.GetFocus();
        if (TargetHandle <> 0) then begin
            Windows.GetCaretPos(CaretPos);
            Windows.GetWindowRect(HotKeyTarget, Rect);

            TargetX := Rect.left + CaretPos.X;
            TargetY := Rect.top + CaretPos.Y;
        end;
        self.ThreadDetach;
    end;
end;





//
// ShowPopupWithFocusRules: Wrapper function to call ShowPopup enforcing the
// focus rules. The rule is to return focus to the original window.
//
// Once the Hotkey has been pressed, you can reshow the
// popup again by calling this procedure.
//
// Program flow continues to a XXXClickEvent routine when the user
// selects an item from the popup window

procedure TFrmMainPopup.ShowPopupWithFocusRules;
begin
    // Attaching to the target keeps the popup from
    // taking away focus from the target item.
    // Example, pasting to the name of a desktop icon. If you remove
    // focus, you remove the rename box

    if (self.PopupUseKB) then begin
        // we need to be foreground for keyboard focus
        Windows.SetForegroundWindow(self.Handle);
        self.ShowPopup(TargetX, TargetY);

        // make sure focus is returned
        if (self.ThreadAttach(self.HotKeyForeWindow)) then begin
            SetForegroundWindow(self.HotKeyForeWindow);
            self.ThreadDetach;
        end;
    end else begin
        if (not self.ThreadAttach(self.HotKeyForeWindow)) then begin
            ShowMessage('ERROR: Couldn''t attach popup menu to this window');
            frmClipboardManager.SetIgnoreClipboard(false);
            exit;
        end;

        self.ShowPopup(TargetX, TargetY);
        self.ThreadDetach();
    end;
end;




procedure TfrmMainPopup.ShowPopup(X,Y: integer);
    //
    // create a 1..9 keys for items 0 to 8 and A to Z keys for items 9 and so no
    //
    function GetAccelerator(i: integer): char;
    begin
        case (i) of
        0..8: result := chr(byte('1') + i);
        else
            result := chr(byte('A') + (i - 9));
        end;
    end;

    // show a ' ...' when item is too long
    // handle tabs  and handle ampersans
    function GetTrimmedItem(item : string) : string;
    begin
        result := item;
        if (length(result) > VisibleCharCount) then begin
            result := leftstr(item, VisibleCharCount) + ' ...';
        end;

        // tabs make the windows too damn long
        // the #10 will make a funky box in the tab's place
        result := stringreplace(result, #9,#10, [rfReplaceAll]);
        result := stringreplace(result, '&', '&&', [rfReplaceAll]);
    end;

    procedure DrawIconOnMenuItem(m : TMenuItem; icon : HICON);
    begin
        if (icon <> 0) then begin
            //DrawIcon(imgIcon.Canvas.Handle,0,0,icon);
            m.bitmap.width := 32;
            m.bitmap.height := 32;
            m.bitmap.pixelformat := pf24bit;
            if not DrawIcon(m.Bitmap.Canvas.Handle,0,0,icon) then begin
                self.AppendLog('DrawIcon failed:' + SysUtils.SysErrorMessage(getlasterror) );
            end;
        end;
    end;

var i: integer;
    m, subm : TMenuItem;
    s: string;
    icon : HICON;
begin
    //
    // Recreate the menuitems
    //
    popup.Items.Clear;

    if (self.PopupUseKB) then begin
        popup.AutoHotkeys := maAutomatic;
    end else begin
        popup.AutoHotkeys := maManual;
    end;

    for i := 0 to (ClipQueue.GetQueueCount - 1) do begin
        m := TMenuItem.Create(popup);
        s := ClipQueue.GetQueueItem(i);
        if (self.PopupUseKB) then begin
            m.Caption  := '&' + GetAccelerator(i) + ': ' + GetTrimmedItem(s);
        end else begin
            m.Caption  :=  GetTrimmedItem(s);
        end;
        m.hint := s;

        icon := ClipQueue.GetQueueIcon(i);
        DrawIconOnMenuItem(m, icon);
        m.OnClick := self.MenuItemClickEvent;
        popup.Items.Add(M);
    end;
    if (self.FullMode) then begin
        m := TMenuItem.Create(Popup);
        m.Caption := 'Flush Text';
        m.OnClick := self.FlushTextItemsClickEvent;
        m.Bitmap := imgFlush.Picture.Bitmap;
        popup.Items.Add(m);
    end;

    if (self.DoShowEditHistory) or (self.FullMode) then begin
        m := TMenuItem.Create(Popup);
        m.Caption := 'Edit History...';
        m.OnClick := self.EditHistoryItemClickEvent;
        m.Bitmap := imgEdit.Picture.Bitmap;
        popup.Items.Add(m);
    end;

    //
    // other clipboard items
    // The hint contains the items index in the queue
    //
    if (OtherQueue.GetCount > 0) then begin
        popup.Items.Add(NewLine());

        for i := (OtherQueue.GetCount - 1) downto 0 do begin
            m := TMenuItem.Create(popup);
            m.Caption := OtherQueue.GetContentName(i) + ': size ' + IntToStr(Trunc(OtherQueue.GetSize(i) / 1024)) + ' KB';
            m.Hint := IntToStr(i);

            DrawIconOnMenuItem(m, OtherQueue.GetIcon(i));
            m.OnClick := OtherItemClickEvent;

            popup.items.Add(m);
        end;

        if (self.FullMode) then begin
            m := TMenuItem.Create(popup);
            m.Caption := 'Flush non-text';
            m.OnClick := FlushOtherItemsClickEvent;

            m.Bitmap := imgFlush.Picture.Bitmap;

            popup.Items.Add(m);
        end;
    end;

    //
    // divider line cancel cancel
    // divider line Last string option
    //
    popup.Items.Add(NewLine());
    m := TMenuItem.Create(popup);
    m.caption := 'Cancel';
    m.OnClick := self.CancelMenuItemClickEvent;
    popup.Items.Add(m);

    if ((self.DoShowLast) or (self.FullMode))and (trim(LastStringSelected) <> '') then begin
        popup.Items.Add(Menus.NewLine());
        m := TMenuItem.Create(popup);
        m.caption := 'Last: ' + GetTrimmedItem(LastStringSelected);
        m.OnClick := self.LastMenuItemClickEvent;

        popup.Items.Add(m);
    end;

    // current clipboard item
    // (may need to add line if 'Last' doesn't exist yet)
    if (self.DoShowCurrentItem)  or (self.FullMode) then begin
        if (trim(LastStringSelected) = '') or (not (self.DoShowLast or FullMode)) then begin
            popup.Items.Add(Menus.NewLine());
        end;
        m := TMenuItem.Create(popup);
        m.Hint := clipboard.AsText;
        m.caption := 'Current: ' + GetTrimmedItem(m.Hint);
        m.OnClick := self.CurrentMenuItemClickEvent;
        popup.Items.Add(m);

    end;

    //
    // paste all / paste selected
    //
    if (self.DoShowPasteAll)  or (self.FullMode) then begin
        popup.Items.Add(menus.NewLine());
        m := TMenuItem.Create(popup);
        m.caption := 'Paste All';
        m.OnClick := self.PasteAllMenuItemClickEvent;
        m.Bitmap := self.imgPasteAll.Picture.Bitmap;
        popup.Items.Add(m);

        m := TMenuItem.Create(popup);
        m.caption := 'Paste Selected...';
        m.OnClick := self.PasteSelectedMenuItemClickEvent;
        m.Bitmap := self.imgPaste.Picture.Bitmap;
        popup.Items.Add(m);

    end;
    //
    // permanent items groups
    //
    if (self.UsePermanentItems) or (self.FullMode)  then begin
        popup.Items.Add(menus.NewLine());
        for i := 0 to frmPermanent.GetCount - 1 do begin
            m := TMenuItem.Create(popup);
            m.hint := frmPermanent.GetItemName(i);
            m.Caption := m.Hint;
            m.OnClick := self.PermanentMenutItemClickEvent;
            popup.Items.Add(m);
        end;

        //
        // don't show in full mode if user specified submenu for Permanent
        // Items
        //
        if (self.DoShowPermanentItemGroups)
            or (self.FullMode and (not self.DoShowPermanentItemSubmenu)) then begin
            s := lowercase(frmPermanent.GetPermanentPath);
            for i := 0 to frmPermanent.PermFoldersGetCount - 1 do begin
                if (lowercase(frmPermanent.PermFoldersGetItem(i)) <> s) then begin
                    m := TMenuItem.Create(popup);
                    m.Hint := frmPermanent.PermFoldersGetItem(i);
                    m.Caption := 'Switch to ' + m.Hint;
                    m.OnClick := self.PermanentGroupItemClickEvent;
                    m.Bitmap := imgRightArrow.Picture.Bitmap;
                    popup.Items.Add(m);
                end;
            end;
        end;
    end;

    //
    // permanent items submenu
    //
    if (self.DoShowPermanentItemSubmenu) then begin
        subm := TMenuItem.Create(popup);
        subm.Caption := 'Permanent Items...';
        subm.Bitmap := imgPermCasc.Picture.Bitmap;
        s := lowercase(frmPermanent.GetPermanentPath);

        for i := 0 to frmPermanent.PermFoldersGetCount - 1 do begin
            if (lowercase(frmPermanent.PermFoldersGetItem(i)) <> s) then begin
                m := TMenuItem.Create(popup);
                m.Hint := frmPermanent.PermFoldersGetItem(i);
                m.Caption := 'Switch to ' + m.Hint;
                m.OnClick := self.PermanentGroupItemClickEvent;

                subm.Add(m);
            end;
        end;

        popup.items.Add(subm);
    end;


    //
    // full mode/ configured mode
    //
    popup.Items.Add(menus.NewLine());
    m := TMenuItem.Create(popup);
    if (self.FullMode = true) then begin
        m.Caption := 'Configured Mode';
        m.Bitmap := self.imgConfMode.Picture.Bitmap;
    end else begin
        m.Caption := 'Full Mode';
        m.Bitmap := self.imgFull.Picture.Bitmap;
    end;
    m.OnClick := self.FullModeMenuItemClickEvent;

    popup.Items.Add(m);

    //
    // show
    // NOTE: This code finishes before .SendText() is ever executed
    //       I need to stop ignoring here because the popup can be dismissed
    //       by ESC key or focus lost.
    AppendLog('Popup Activated');
    popup.Popup(x, y);

    AppendLog('Popup End');
    frmClipboardManager.SetIgnoreClipboard(false);
end;





{
--============================
-- // Popup MenuItem Clicks //
--============================

Description:

Handle a normal menu item,
other item,
'cancel' item,
Last item,
Current item,
Paste All,
Paste Selected,
Permanent item

Text item make use of the SendText routing to enforce configuration
option.

Remember the last selected text item for the 'last' menu item option.

Inform the clipboard manager of any pasting we do.
}


//
// KeyPressedOverride
// Returns TRUE when an alternate action has taken place
// because of a KEYSTROKE+Click action by the user
// Returns FALSE when no override took place
//
function TFrmMainPopup.KeyPressedOverride(s : string) : boolean;
    //
    // run text as shell command
    //
    function CTRLOpen(s : string) : boolean;
    begin
        result := true;

        // detect held CTRL key
        // Execute item or show menu
        if (KeyboardQuery.IsPressed(VK_CONTROL)) then begin
            if (self.UseCtrlClickMenu) then begin
                if (self.ThreadAttach(self.HotKeyForeWindow)) then begin
                    Windows.SetForegroundWindow(self.Handle);
                    self.ThreadDetach;
                end;

                pmItemOptions.Items.RethinkHotkeys;
                pmItemOptions.Items[0].Caption := '&1  Run "' + leftstr(self.MenuItemText, self.VisibleCharCount)
                    + '..."';
                pmItemOptions.Popup(self.TargetX, self.TargetY );

                // make sure focus is returned
                if (self.ThreadAttach(self.HotKeyForeWindow)) then begin
                    SetForegroundWindow(self.HotKeyForeWindow);
                    self.ThreadDetach;
                end;
            end else begin
                self.Run1.Click;
                {ShellAPI.ShellExecute(self.Handle,nil,PChar(s),nil,nil,SW_NORMAL);
                frmClipboardManager.SetIgnoreClipboard(false); // just incase
                }
            end;
            EXIT;
        end;
        result := false;
    end;


    //
    //  Show a preview of item when SHIFT-Clicked
    //
    function SHIFTOpen(s : string) : boolean;
    begin
        result := true;
        if (KeyboardQuery.IsPressed(VK_SHIFT)) then begin
            self.Preview1.Click;
            {FrmPreview.SetText(s);
            self.ShowPreviewForm;
            }
            EXIT;
        end;
        result := false;
    end;

    function CopyKeyOpen(s : string) : boolean;
    begin
        //
        // just turn on "Paste Only" toggle and let
        // selected menu item handle sending the text as normal
        //
        if (KeyboardQuery.IsPressed(self.CopyKey)) then begin
            Self.ClipboardOnlyWindow := true;
        end;
        result := false;
    end;

    function PermanentKeyOpen(s : string) : boolean;
    begin
        //
        // Show Permanent Items dialog with new entry
        //
        result := true;
        if (KeyboardQuery.IsPressed(Self.PermanentKey)) then begin
            Windows.SetForegroundWindow(FrmPermanent.Handle);
            frmPermanent.ShowWithNewItem(s);
            EXIT;
        end;
        result := false;
    end;

    function MimicKeyOpen(s : string) : boolean;
    begin
        // just turn on "Mimic Typing" toggle and let
        // selected menu item handle sending the text as normal
        if (KeyboardQuery.IsPressed(self.MimicKey)) then begin
            self.MimicTypingOnce := true;
        end;
        result := false;
    end;
begin
    result := CTRLOpen(s);
    if (result) then EXIT;

    result := SHIFTOpen(s);
    if (result) then EXIT;

    result := CopyKeyOpen(s);
    if (result) then EXIT;

    result := MimicKeyOpen(s);
    if (result) then EXIT;

    result := PermanentKeyOpen(s);
    if (result) then EXIT;

end;

procedure TFrmMainPopup.ShowPreviewForm;
begin
    Windows.SetForegroundWindow(FrmPreview.Handle);

    FrmPreview.Left := Self.TargetX;
    FrmPreview.Top := Self.TargetY;
    FrmPreview.ShowModal;
    self.ShowPopupWithFocusRules;
end;


procedure TfrmMainPopup.MenuItemClickEvent(Sender: TObject);
var s : string;
begin
    // paste item
    with Sender as TMenuItem do begin
        s := Hint;
        MenuItemText := s;
        if (self.KeyPressedOverride(s)) then begin
            EXIT;
        end;

        LastStringSelected := s;
        if (self.DoMoveSelectedToTop) then
            ClipQueue.MoveToStart(MenuIndex);


        AppendLog('Sending text---: ' + s);
        self.SendText(s);
        AppendLog('inserting done');
    end;
end;

procedure TFrmMainPopup.FlushTextItemsClickEvent(Sender: TObject);
begin
    ClipQueue.ClearQueue;
    self.FullMode := false;
    self.ShowPopupWithFocusRules;
end;

procedure TFrmMainPopup.EditHistoryItemClickEvent(Sender : TObject);
begin
    //
    // show the edit window
    // re-show the popup when the edit window it closed
    //
    Windows.SetForegroundWindow(frmClipboardManager.Handle);
    frmClipboardManager.ShowModal;
    self.ShowPopupWithFocusRules;
end;

procedure TFrmMainPopup.OtherItemClickEvent(Sender: TObject);
var s : string;
begin
    //
    // the Hint contains the OtherQueue item's index
    //
    with Sender as TMenuItem do begin
        s := Hint;

        // preview only
        if (KeyboardQuery.IsPressed(VK_SHIFT)) then begin
            FrmPreview.SetPicture;
            OtherQueue.GetDIB(StrToInt(s), FrmPreview.imgPreview.picture);

            self.ShowPreviewForm;
            EXIT;
        end;

        // Copy - with no paste 
        if (KeyboardQuery.IsPressed(VK_F1)) then begin
            // try to clear phantom keystroke from entering other program
            keybd_event(VK_F1, VkKeyScan(char(VK_F1)), KEYEVENTF_KEYUP, 0);
            self.ClipboardOnlyWindow := true;
        end;

        //
        // normal paste
        frmClipboardManager.SetIgnoreClipboard(true);
        OtherQueue.SetClipboardTo(StrToInt(s));

        if (UseOtherPasting) and (not ClipboardOnlyWindow)then begin
            self.SendCTRL_V;
            frmClipboardManager.InformOfPaste;
        end;
        frmClipboardManager.SetIgnoreClipboard(false);
        AppendLog('Placing other item on clipboard: index=' + s);

        // try to clear phantom keystroke from entering other program
        keybd_event(VK_F1, VkKeyScan(char(VK_F1)), KEYEVENTF_KEYUP, 0);
    end;
end;

procedure TFrmMainPopup.FlushOtherItemsClickEvent(Sender: TObject);
begin
    OtherQueue.ClearQueue;
    self.FullMode := false;
    self.ShowPopupWithFocusRules;
end;


procedure TFrmMainPopup.CurrentMenuItemClickEvent(Sender: TObject);
var s : string;
begin
    with Sender as TMenuItem do begin
        s := Hint;
        if (self.KeyPressedOverride(s)) then begin
            EXIT;
        end;

        LastStringSelected := s;
        AppendLog('Sending text---: ' + s);
        self.SendText(s);
        AppendLog('inserting done');
    end;
end;

procedure TfrmMainPopup.LastMenuItemClickEvent(Sender: TObject);
begin
    AppendLog('Inserting last selected string');
    if (self.KeyPressedOverride(LastStringSelected)) then begin
        EXIT;
    end;
    self.SendText(LastStringSelected);
end;

procedure TfrmMainPopup.CancelMenuItemClickEvent(Sender: TObject);
begin
    //
    // since using the keyboard takes away focus from the target
    // window, it must be returned
    //
    if (self.PopupUseKB) then begin
        if (self.ThreadAttach(self.HotKeyForeWindow)) then begin
            Windows.SetForegroundWindow(self.HotKeyForeWindow);
            self.ThreadDetach;
        end;
    end;
end;

procedure TfrmMainPopup.PermanentMenutItemClickEvent(sender: TObject);
var s : string;
begin
    with sender as tmenuitem do begin
        s := frmPermanent.GetTextFrom( Hint );
        MenuItemText := s;
        if (self.KeyPressedOverride(s)) then begin
            EXIT;
        end;

        self.SendText( s );
    end;
end;

procedure TFrmMainPopup.PermanentGroupItemClickEvent(sender: TObject);
var s : string;
begin
    with sender as tmenuitem do begin
        s := Hint;
        frmPermanent.SetPermanentPath(s);

        self.ShowPopupWithFocusRules;
    end;
end;

procedure TfrmMainPopup.PasteAllMenuItemClickEvent(sender: TObject);
var i : longint;
    s : string;
begin
    s := '';
    for i := 0 to (ClipQueue.GetQueueCount  - 1) do begin
        if (Cardinal(i) = (ClipQueue.GetQueueCount  - 1)) then begin
            s := s + ClipQueue.GetQueueItem(i);
        end else begin
            s := s + ClipQueue.GetQueueItem(i) + #13#10;
        end;
    end;

    self.SendText(s);
end;

procedure TfrmMainPopup.PasteSelectedMenuItemClickEvent(sender: TOBject);
var s : string;
begin
    s := FrmPasteSelected.GetSelectedItems;
    // the above windows takes away focus from target
    // here, so it needs it back before pasting
    if (not self.PopupUseKB) then begin
        self.ThreadAttach(self.HotKeyForeWindow);
        Windows.SetForegroundWindow(self.HotKeyForeWindow);
        self.ThreadDetach;
    end;

    if (s <> '') then begin
        self.SendText(s);
    end else begin
        self.CancelMenuItemClickEvent(self);
    end;
end;


procedure TFrmMainPopup.FullModeMenuItemClickEvent(Sender : TObject);
begin
    //
    // toggle Full->Configured and Configured->Full
    //
    self.FullMode := self.FullMode xor true;
    self.ShowPopupWithFocusRules;
end;


//-------------------------------------------------------------------
// fake a CTRL+V, fake keyboard typing, or only place on clipboard
//-------------------------------------------------------------------

procedure TfrmMainPopup.SendText(s: string);
    procedure SendUsingKeyboardMimic(s : string);
    var c : char;
        w : word;
        i : integer;
        ShiftPressed, EnterPressed : boolean;
    begin
        self.AppendLog('mimic typing');
        for i := 1 to length(s) do begin
            c := s[i];
            w := VkKeyScan(c);
            ShiftPressed := (hi(w) and 1) > 0;
            EnterPressed := (byte(c) = VK_RETURN);
            {VkKeyScan: The first bit of the hi byte set means shift is pressed}

            {Ditch LF - assume CR came first}
            if (c <> #10) then begin
                if ShiftPressed and (not EnterPressed) then begin
                    keybd_event(VK_RSHIFT, 0, 0, 0);
                end;

                {Press and release key}
                keybd_event(lo(w), w, 0, 0);
                keybd_event(lo(w), w, KEYEVENTF_KEYUP, 0);

                if ShiftPressed and (not EnterPressed) then begin
                    keybd_event(VK_RSHIFT, 0, KEYEVENTF_KEYUP, 0);
                end;
            end;

            if (i mod 10 = 0) then sleep(1); // give the keyboard buffer a little break
        end;
    end;
    procedure SendUsingPaste(s : string);
    begin
        //
        // place text on clipboard and paste via CTRL+P
        //

        AppendLog('clearing and placing selected text on clipboard');
        AppendLog('---"' + s + '"---');
        Clipboard.SetTextBuf(PChar(s));

        sleep(10);
        if (self.ShiftInsertOnce) then begin
            Self.SendSHIFT_INSERT;
        end else if (self.CtrlVOnce) then begin
            Self.SendCTRL_V;
        end else if (self.UsePasting)  then begin
            Self.SendCTRL_V;
        end else if (self.UsePastingSI) then begin
            Self.SendSHIFT_INSERT;
        end;
    end;

begin
    AppendLog('[Paste Start]');

    frmClipboardManager.SetIgnoreClipboard(true);

    //
    // Since the popup may have stolen focus from the target,
    // it must be given keyboard focus again
    //
    if (self.PopupUseKB) then begin
        if (self.ThreadAttach(self.HotKeyForeWindow)) then begin
            Windows.SetForegroundWindow(self.HotKeyForeWindow);
            Windows.SetFocus(self.HotKeyTarget);
        end else begin
            showmessage('ERROR: Unable to paste into target');
            exit;
        end;
    end;

    // send text using configured method
    // Update: ClipboardOnlyWindow will override and force
    //          a clipboard only operation

    if (ClipboardOnlyWindow) then begin
        clipboard.SetTextBuf(PChar(s));
    end else if (self.UseKeyboardMimic or self.MimicTypingOnce) then begin
        SendUsingKeyboardMimic(s);
        self.MimicTypingOnce := false;
    end else if (self.UsePasting or self.UsePastingSI) then  begin
        SendUsingPaste(s);
        self.CtrlVOnce := false;
        self.ShiftInsertOnce := false;
    end else begin
        // send clipboard only
        clipboard.SetTextBuf(PChar(s));
    end;

    if (self.PopupUseKB) then begin
        self.ThreadDetach;
    end;


    AppendLog('[Paste End]');
    frmClipboardManager.InformOfPaste;
    frmClipboardManager.SetIgnoreClipboard(false);
end;


//
// used by SendText and OtherItemClickEvent to simulate the user
// pressing CTRL+V to paste
//
procedure TFrmMainPopup.SendCTRL_V;
var w : word;
begin
    AppendLog('sending CTRL+V');

    // clear any phantom keystrokes
    keybd_event(VK_SHIFT, VkKeyScan(char(VK_SHIFT)), KEYEVENTF_KEYUP, 0);
    keybd_event(VK_CONTROL , VkKeyScan(char(VK_control)), KEYEVENTF_KEYUP, 0);
    sleep(10);

    // press CTRL, press V, release V, release CTRL
    keybd_event(VK_CONTROL , VkKeyScan(char(VK_control)), 0, 0);
    sleep(10);
            w := VkKeyScan('V');
            keybd_event(lo(w), w, 0, 0);
            sleep(10);

            keybd_event(lo(w), w, KEYEVENTF_KEYUP, 0);
            sleep(10);
    keybd_event(VK_CONTROL , VkKeyScan(char(VK_control)), KEYEVENTF_KEYUP, 0);
    sleep(10);
    AppendLog('sent CTRL+V');
end;

procedure TFrmMainPopup.SendSHIFT_INSERT;
begin
    AppendLog('sending SHIFT+INSERT');

    // clear any phantom keystrokes
    keybd_event(VK_SHIFT, VkKeyScan(char(VK_SHIFT)), KEYEVENTF_KEYUP, 0);
    keybd_event(VK_CONTROL , VkKeyScan(char(VK_control)), KEYEVENTF_KEYUP, 0);
    sleep(10);

    // press SHIFT, press INSERT, release INSERT, release SHIFT

    keybd_event(VK_RSHIFT,0,0,0);
    sleep(10);
            keybd_event(VK_INSERT,0,0,0);
            sleep(10);

            keybd_event(VK_INSERT,0,KEYEVENTF_KEYUP,0);
            sleep(10);

    keybd_event(VK_RSHIFT,0,KEYEVENTF_KEYUP,0);
    sleep(10);
    AppendLog('sent SHIFT+INSERT');
end;



//---------------------------------------------------------------------------
// Ultra-Dirty trick
// I'm attaching a [null]ArsClip[null] signature to each string.
// When converted by each program, only characters before the null
// will be seen as string data.
//
// This will let my program know that it put the string on the clipboard
// durring a paste operation
//---------------------------------------------------------------------------
{procedure TFrmMainPopup.PlaceTextOnClipboard(s : string);
    procedure SetBuffer(format: Word; var buffer; Size: Integer);
    var Data: THandle;
        DataPtr: Pointer;
    begin
        clipboard.Open;
        try
            Data := Windows.GlobalAlloc(GMEM_MOVEABLE+GMEM_DDESHARE, Size);
            try
                DataPtr := GlobalLock(Data);
                try
                    Move(Buffer, DataPtr^, Size);
                    clipboard.Clear;
                    Windows.SetClipboardData(Format, Data);
                finally
                    Windows.GlobalUnlock(Data);
                end;
            except
                Windows.GlobalFree(Data);
                raise;
            end;
        finally
            clipboard.Close;
        end;
    end;

begin

    s := s + #0 + 'ArsClip' + #0;
    SetBuffer(CF_TEXT, PChar(s)^, length(s) + 1);
end;
}


{
--============================
-- // Thead utility methods //
--============================

Description: Mimic being part of the targeted window. This method is used
to get the keyboard focused item of an outside process and to associate a
popup menu with an outside process.

}

function TfrmMainPopup.ThreadAttach(TargetWindow: cardinal) : boolean;
begin
    result := false;
    If (ThreadAttached) then begin
        showmessage('Error: Thread already attached');
        self.Close;
    end;

    ThreadTarget := Windows.GetWindowThreadProcessId(TargetWindow, nil);
    ThreadOurs := Windows.GetCurrentThreadId();
    if (ThreadTarget <> ThreadOurs) then begin
        result := Windows.AttachThreadInput(ThreadTarget, ThreadOurs, true);
        ThreadAttached := true;
    end;
end;
procedure TfrmMainPopup.ThreadDetach();
begin
    If (not ThreadAttached) then begin
        showmessage('Error: Thread not attached');
        self.Close;
    end;

    if (ThreadTarget <> ThreadOurs) then begin
        Windows.AttachThreadInput(ThreadTarget, ThreadOurs, false);
        ThreadAttached := false;
    end;
end;

{
--===========
-- // Etc. //
--===========
}
procedure TfrmMainPopup.AppendLog(s : string);
begin
    if length(memo1.text) > 20000 then memo1.Text := '';
    // Cheezy Debugging Info[tm]
    // make sure we don't overflow the text, only accept 1000 characters
    // at a time
    memo1.Text :=  leftstr(s, 1000) + #13 + #10 + memo1.text;
end;

procedure TfrmMainPopup.btnHideClick(Sender: TObject);
begin
    self.Hide;
end;

{
--====================
-- CTRL option menu items
--===================
}
procedure TFrmMainPopup.Run1Click(Sender: TObject);
var s : string;
begin
    if ShellAPI.ShellExecute(self.Handle,nil,
        PChar(self.MenuItemText),nil,nil,SW_NORMAL) <= 32 then begin
        s := trim(self.MenuItemText);
        if (pos('@',s) <> 0) then begin
            s := 'mailto:' + s;
            shellAPI.ShellExecute(self.Handle,nil,
                PChar(s),nil,nil,SW_NORMAL)
        end;
    end;
    frmClipboardManager.SetIgnoreClipboard(false); // just incase
end;

procedure TFrmMainPopup.MakePermanentItem1Click(Sender: TObject);
begin
    Windows.SetForegroundWindow(FrmPermanent.Handle);
    frmPermanent.ShowWithNewItem(self.MenuItemText);
end;

procedure TFrmMainPopup.PasteusingMimicTyping1Click(Sender: TObject);
begin
    self.MimicTypingOnce := true;
    self.SendText(self.MenuItemText);
end;

procedure TFrmMainPopup.Placeonclipboard1Click(Sender: TObject);
begin
    self.ClipboardOnlyWindow := true;
    self.SendText(self.MenuItemText);
end;

procedure TFrmMainPopup.Preview1Click(Sender: TObject);
begin
    FrmPreview.SetText(self.MenuItemText);
    self.ShowPreviewForm;
end;

procedure TFrmMainPopup.back1Click(Sender: TObject);
begin
    self.ShowPopupWithFocusRules;
end;

procedure TFrmMainPopup.PasteusingSHIFTINS1Click(Sender: TObject);
begin
    self.ShiftInsertOnce := true;
    self.SendText(self.MenuItemText);
end;

procedure TFrmMainPopup.PasteusingCTRLV1Click(Sender: TObject);
begin
    self.CtrlVOnce := true;
    self.SendText(self.MenuItemText);
end;

end.
