unit UnitFrmPermanentNew;
{
    Purpose:
        This unit stores/reads/edits the permanent items.
        The form is not a dummy form.


    Updates:
        Complete Rewrite. Old form was too poorly designed
        to save.
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

const DEFAULT_FOLDER = 'Default';
      ADDNEW_FOLDER = '<add new>';
      PERM0_FILE = 'perm0.ini';
      PERM1_FILE = 'perm1.ini';

type
  TFrmPermanent = class(TForm)
    cbPIGs: TComboBox;
    btnAddPIG: TButton;
    btnDeletePIG: TButton;
    Panel1: TPanel;
    btnUp: TButton;
    btnDown: TButton;
    btnDelete: TButton;
    lbItemNames: TListBox;
    Label1: TLabel;
    Panel2: TPanel;
    Label2: TLabel;
    txtItemName: TEdit;
    Label3: TLabel;
    mItemText: TMemo;
    Label4: TLabel;
    btnEdit: TButton;
    btnSave: TButton;
    btnCancel: TButton;
    btnNew: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure cbPIGsChange(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
    procedure btnUpClick(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure lbItemNamesClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnAddPIGClick(Sender: TObject);
    procedure btnNewClick(Sender: TObject);
    procedure btnDeletePIGClick(Sender: TObject);
  private
  { Private declarations }
        AppPath : string;
        CurrentPermPath : string;
        FolderList : TStringList;
        ItemNameList : TStringList;
        ItemDataList : TStringList;

        // util functions
        function GetOldDataFilename (i: integer): string;
        function GetDataFilename (i: integer): string;
        procedure UpdateFolderList;
        procedure LoadPermanentItemsGroup;
        procedure SavePermanentItemsGroup;
        function IsPermanentPathFolder(folder : string) : boolean;
        procedure RefreshFormData;
        //
        procedure SetEditMode(value : boolean);
        procedure ShowSelectedItem;
  public
  { Public declarations }
        procedure SetPermanentPath( path : string );
        function GetPermanentPath : string;

        // to enumerate permanent item names
        function PermFoldersGetCount : cardinal;
        function PermFoldersGetItem(index : cardinal) : string;

        // to enumerate items in the current permanent folder
        function GetCount: integer;
        function GetItemName(i: integer): string;
        function GetItemText(i: integer): string;
        function GetTextFrom(name: string): string;

        // display form with a newly created item
        procedure ShowWithNewItem(item : string);
  end;

var
  FrmPermanent: TFrmPermanent;

implementation

{$R *.dfm}


//
// public API
//
procedure TfrmPermanent.SetPermanentPath( path : string );
begin
    if Self.IsPermanentPathFolder(path) then  begin
        CurrentPermPath := path;
        self.LoadPermanentItemsGroup;
    end else begin
        CurrentPermPath := DEFAULT_FOLDER;
        self.LoadPermanentItemsGroup;
    end;
end;

function TfrmPermanent.GetPermanentPath : string;
begin
    result := self.CurrentPermPath;
end;

function TfrmPermanent.PermFoldersGetCount : cardinal;
begin
    result := FolderList.Count;
end;
function TfrmPermanent.PermFoldersGetItem(index : cardinal) : string;
begin
    result := FolderList[index];
end;


function TfrmPermanent.GetCount: integer;
begin
    result := ItemNameList.Count;
end;
function TfrmPermanent.GetItemName(i: integer): string;
begin
    result := ItemNameList[i];
end;
function TfrmPermanent.GetItemText(i: integer): string;
begin
    result := ItemDataList[i];
end;
function TfrmPermanent.GetTextFrom(name: string): string;
var pos: integer;
    i: integer;
begin
    pos := -1;
    for i := 0 to ItemNameList.count - 1 do begin
        if (lowercase(name) = lowercase(ItemNameList[i])) then begin
            pos := i;
        end;
    end;

    result := ItemNameList[pos];
end;


procedure TfrmPermanent.ShowWithNewItem(item : string);
begin
    self.Show;
    self.btnNew.Click;
    mItemText.text := item;
end;


//
// Create / Destroy
//

procedure TFrmPermanent.FormCreate(Sender: TObject);
var    name: string;
begin
    self.FolderList := TStringList.Create;
    self.ItemNameList := TStringList.Create;
    self.ItemDataList := TStringList.Create;

    self.CurrentPermPath := DEFAULT_FOLDER;
    self.AppPath := ExtractFilePath(application.ExeName);


    //
    // make the new Default directory and import and
    // current permanent items
    //
    if not DirectoryExists( self.AppPath + DEFAULT_FOLDER) then begin
        mkdir(self.AppPath + DEFAULT_FOLDER);

        name := GetOldDataFilename(0);
        if FileExists(name) then
            copyfile(pchar(name), PChar(GetDataFileName(0)), true);

        name := GetOldDataFilename(1);
        if fileExists(name) then
            copyfile(pchar(name), PChar(GetDataFilename(1)), true);
    end;

    self.UpdateFolderList;
end;


//
// Util Functions
//

function TfrmPermanent.GetOldDataFilename(i: integer): string;
begin
    result := self.AppPath + 'perm' + IntToStr(i) + '.ini';
end;
function TfrmPermanent.GetDataFilename(i: integer): string;
begin
    case i of
    0: result := IncludeTrailingPathDelimiter(self.AppPath + CurrentPermPath) + PERM0_FILE;
    1: result := IncludeTrailingPathDelimiter(self.AppPath + CurrentPermPath) + PERM1_FILE;
    end
end;

procedure TfrmPermanent.LoadPermanentItemsGroup;
var name, itemText, s : string;
    lineCount : cardinal;
    i : integer;
    tf : textfile;
begin
    //
    // load permanent items
    //
    ItemNameList.Create;
    name := GetDataFilename(0);
    if FileExists(name) then begin
        ItemNameList.LoadFromFile(name);
    end;

    //
    // abort reading and show message on error
    // always close the file
    //
    ItemDataList.Clear;
    name := GetDataFilename(1);
    if FileExists(name) then begin
        AssignFile(tf, name);
        Reset(tf, name);

        try
            while not eof(tf) do begin
                try
                    Readln(tf, s);
                    itemText := '';
                    lineCount := StrToInt(s);

                    for i := 0 to lineCount - 1 do begin
                        Readln(tf, s);
                        if (itemText = '') then begin
                            itemText := s;
                        end else begin
                            itemText := itemText + chr(13) + chr(10) + s;
                        end;
                    end;

                    ItemDataList.Add(itemText);
                except
                     on E: Exception do begin
                        ShowMessage('The "Permanent Item" file for group ' + CurrentPermPath + ' is corrupted - ' + name + #13#10#13#10 +
                                    'Error Message: ' + E.Message);
                        break;
                     end;
                end;
            end;
        finally
            CloseFile(tf);
        end;
    end;
end;

procedure TfrmPermanent.SavePermanentItemsGroup;
var name: string;
    s : string;
    cnt : cardinal;

    i,j: longint;
    tf: textfile;
begin
    if (CurrentPermPath = '') then
        EXIT;

    if not DirectoryExists(IncludeTrailingPathDelimiter(AppPath) + CurrentPermPath) then begin
        mkdir(IncludeTrailingPathDelimiter(AppPath) + CurrentPermPath);
    end;
    //
    // save items
    //
    name := GetDataFilename(0);
    ItemNameList.SaveToFile(name);

    name := GetDataFilename(1);
    AssignFile(tf, name);
    Rewrite(tf);


    for i := 0 to ItemDataList.Count - 1 do begin
        s := ItemDataList[i];

        cnt := 1;
        for j := 1 to length(s) - 1 do begin
            if (s[j] = #13) and (s[j+1]= #10) then inc(cnt);
        end;

        writeln(tf, cnt);
        writeln(tf, s);
    end;

    CloseFile(tf);
end;


procedure TfrmPermanent.UpdateFolderList;
var rec : TSearchRec;
    r : integer;
begin
    //
    // scan each subfolder and look for permanent item config files
    // this will generate a list of permanent item groups (using their
    // folder name)
    //
    FolderList.clear;
    r := FindFirst(AppPath + '*.*', faDirectory, rec);
    while (r = 0) do begin
        // is file a subfolder?
        if (rec.Attr and faDirectory) > 0 then begin
            if (rec.name <> '.') and (rec.name <> '..') then begin
                if IsPermanentPathFolder(rec.name) then begin
                    FolderList.Add(rec.name);
                end;
            end;
        end;

        r := FindNext(rec);
    end;
end;

function TFrmPermanent.IsPermanentPathFolder(folder : string) : boolean;
begin
    result := fileexists(IncludeTrailingPathDelimiter(AppPath) + folder + '\' + PERM0_FILE )
end;


//===================================================
// User Interface Interaction
// [State Philosophy]
// Save after any action that changes the current group.
// This includes item position, additions, or deletions.
//===================================================
procedure TFrmPermanent.FormShow(Sender: TObject);
begin
    self.LoadPermanentItemsGroup;
    self.RefreshFormData;
    self.SetEditMode(false);
    txtItemName.Text := '';
    mItemText.Text := '';
end;

procedure TFrmPermanent.RefreshFormData;
var i : integer;
begin
    // show the curent Permanent Item Group names and
    // select the current folder in the dropdown
    cbpigs.Clear;
    cbPIGs.Items.AddStrings(FolderList);
    for i := 0 to FolderList.Count - 1 do begin
        if lowercase(cbpigs.Items[i]) = lowercase(CurrentPermPath) then begin
            cbPIGs.ItemIndex := i;
        end;
    end;

    // load the current item names
    //
    lbItemNames.Clear;
    lbItemNames.Items.AddStrings(ItemNameList);

end;


//
// group changed or item clicked
//
procedure TFrmPermanent.cbPIGsChange(Sender: TObject);
begin
    //self.SavePermanentItemsGroup;
    self.SetPermanentPath(cbpigs.text);
    self.RefreshFormData;
    txtItemName.Text := '';
    mItemText.Text := '';
end;

procedure TFrmPermanent.lbItemNamesClick(Sender: TObject);
begin
    self.ShowSelectedItem;
end;

// permanent item edit buttons
//
procedure TfrmPermanent.btnUpClick(Sender: TObject);
var i: integer;
begin
    i := lbItemNames.ItemIndex;
    if (i <> 0) then begin
        ItemNameList.Move(i, i - 1);
        ItemDataList.Move(i, i - 1);
        lbItemNames.Items.Move(i, i - 1);

        lbItemNames.ItemIndex := i - 1;
    end;

    self.SavePermanentItemsGroup;
end;
procedure TfrmPermanent.btnDownClick(Sender: TObject);
var i: integer;
begin
    i := lbItemNames.ItemIndex;
    if (i <> lbItemNames.Count -1) then begin
        ItemNameList.Move(i, i + 1);
        ItemDataList.Move(i, i + 1);
        lbItemNames.Items.Move(i, i + 1);

        lbItemNames.ItemIndex := i + 1;
    end;

    self.SavePermanentItemsGroup;
end;

procedure TfrmPermanent.btnDeleteClick(Sender: TObject);
var i: integer;
begin
    i := lbItemNames.ItemIndex;
    if (i <> -1) then begin
        ItemNameList.Delete(i);
        ItemDataList.Delete(i);
        lbItemNames.DeleteSelected;
    end;

    self.SavePermanentItemsGroup;
end;

procedure TFrmPermanent.btnEditClick(Sender: TObject);
var i: integer;
begin
    i := lbItemNames.ItemIndex;
    if (i <> -1) then begin
        ShowSelectedItem;
        SetEditMode(true);
    end;
end;


procedure TfrmPermanent.SetEditMode(value : boolean);
begin
    if (value) then begin
        txtItemName.Enabled := true;
        mItemText.Enabled := true;
        btnSave.Visible := true;
        btnCancel.Visible := true;

        lbItemNames.Enabled := false;
        cbPIGs.Enabled := false;
    end else begin
        txtItemName.Enabled := false;
        mItemText.Enabled := false;
        btnSave.Visible := false;
        btnCancel.Visible := false;

        lbItemNames.Enabled := true;
        cbPIGs.Enabled := true;
    end;
end;

procedure TfrmPermanent.ShowSelectedItem;
var i : integer;
begin
    i := lbItemNames.ItemIndex;
    if (i <> -1) then begin
        txtItemName.Text := ItemNameList[i];
        mItemText.Text := ItemDataList[i];
    end;
end;


//
// Save / Cancel button used durring Edit Mode 
//
procedure TFrmPermanent.btnCancelClick(Sender: TObject);
begin
    self.ShowSelectedItem;
    self.SetEditMode(false);
end;

procedure TFrmPermanent.btnSaveClick(Sender: TObject);
var i : integer;
begin
    if txtItemName.text = '' then begin
        ShowMessage('A name is required for an item');
        EXIT;
    end;
    // detect 'New' or 'Edit'
    i := lbItemNames.ItemIndex;
    if (i = -1) then begin
        ItemNameList.Add(txtItemName.Text);
        ItemDataList.Add(mItemText.text);
    end else begin
        ItemNameList[i] := txtItemName.Text;
        ItemDataList[i] := mItemText.text;
    end;
    self.SavePermanentItemsGroup;
    self.RefreshFormData;

    lbItemNames.ItemIndex := i;
    self.ShowSelectedItem;
    self.SetEditMode(false);
end;


procedure TFrmPermanent.btnNewClick(Sender: TObject);
begin
    self.SetEditMode(true);
    txtItemName.text := '';
    mItemText.text := '';
    self.txtItemName.SetFocus;
    lbItemNames.ItemIndex := -1;
end;

//
// add / delete Permanent Item Group
//
procedure TFrmPermanent.btnAddPIGClick(Sender: TObject);
var newgroup : string;
begin
    if
    Dialogs.InputQuery('Add Group','Please enter a new group name', newgroup)
    then begin
        if not IsPermanentPathFolder(newgroup) then begin
            self.CurrentPermPath  := newgroup;
            self.ItemNameList.Clear;
            self.ItemDataList.clear;
            self.SavePermanentItemsGroup;
            self.UpdateFolderList;
            self.RefreshFormData;
        end else begin
            Dialogs.showmessage('Group name already exists');
        end;
    end;
end;


procedure TFrmPermanent.btnDeletePIGClick(Sender: TObject);
begin
    if DirectoryExists(IncludeTrailingPathDelimiter(AppPath) + cbPIGs.Text) then begin
        deletefile( GetDataFilename(0) );
        deleteFile( GetDataFilename(1) );
        RmDir(IncludeTrailingPathDelimiter(AppPath) + cbPIGs.Text);
        self.SetPermanentPath(cbPIGs.Items[0]);
        self.UpdateFolderList;
        self.RefreshFormData;
    end;
end;

end.
