unit UnitFrmSysTrayMenu;
{
    Purpose:
        This unit is responsible for the system tray icon and it's
        popup menu.
    Updates:
        Updated method of modifying the systray icon so it does change
        the systray or move in position.

        Added public method to disable/enable systray icon changing

        Figured out a simple/reliable way to show picture on menu item
        Using UnitClipQueue
}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ExtCtrls, ShellAPI, StdCtrls, ImgList ;


const MY_WM_TRAYICON = WM_USER + 1; // used to receive tray icon messsages

type
  TfrmSysTrayMenu = class(TForm)
    Label1: TLabel;
    ToggleTimer: TTimer;
    ImageList1: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ToggleTimerTimer(Sender: TObject);
  private
    { Private declarations }
    TrayPopup : TPopupMenu;
    TrayIcon: TNotifyIconData;          {tray icon info}
    iconPic : TIcon;
    DisableIconChanges : boolean;
    procedure ShowPopup;

    procedure ExitMenuItemClickEvent(Sender: TObject);
    procedure FlushHistoryMenuItemClickEvent(sender : TObject);
    procedure EditHistoryMenuItemClickEvent(sender : TObject);
    procedure RemovedItemsMenuItemClickEvent(sender : Tobject);
    procedure SwitchPermFolder(sender : TObject);
    procedure ConfigureMenuItemClickEvent(Sender : TObject);
    procedure RefreshMenuItemClickEvent(Sender : TObject);
    procedure SwitchMenuItemClickEvent(sender : TOBject);
    procedure AboutClickEvent(sender : TObject);
    procedure SetIconByIndex(index : longint);
  public
    { Public declarations }

     procedure WMTrayIcon(var Msg: TMessage); message MY_WM_TRAYICON;
        procedure SetIconNormal;
        procedure SetIconIgnore;
        procedure SetIconTriggered;
        procedure SetDisableIconChanges(value : boolean);
  end;

var
  frmSysTrayMenu: TfrmSysTrayMenu;

implementation

uses UnitFrmPermanent, UnitFrmConfig, UnitFrmClipboardManager,
  UnitFrmMainPopup, UnitClipQueue, UnitFrmAbout, UnitFrmRemoved,
  UnitKeyboardQuery, UnitFrmPermanentNew;
{$R *.dfm}

{////////////////////}
{//Public Interface//}
{////////////////////}

procedure TFrmSysTrayMenu.SetDisableIconChanges(value: boolean);
begin
   self.DisableIconChanges := value;
end;

procedure TFrmSysTrayMenu.SetIconNormal;
begin

    ToggleTimer.Enabled := false;
    if (not self.DisableIconChanges) then begin
        TrayIcon.cbSize := SizeOf(TrayIcon);
        TrayIcon.Wnd := Self.Handle;
        TrayIcon.uID := 0;
        TrayIcon.uFlags := NIF_ICON or NIF_TIP or NIF_MESSAGE;
        TrayIcon.uCallbackMessage := MY_WM_TRAYICON;
        TrayIcon.hIcon := Application.Icon.Handle;
        TrayIcon.szTip := 'ArsClip';
        ShellAPI.Shell_notifyIcon(NIM_MODIFY, @TrayIcon);
    end;
end;

procedure TFrmSysTrayMenu.SetIconIgnore;
begin
    self.SetIconByIndex(0);
end;

procedure TFrmSysTrayMenu.SetIconTriggered;
begin
    self.SetIconByIndex(1);
end;

procedure TFrmSysTrayMenu.SetIconByIndex(index : longint);
begin
    ToggleTimer.Enabled := true;

    if (not self.DisableIconChanges) then begin
        ImageList1.GetIcon(index, IconPic);
        TrayIcon.cbSize := SizeOf(TrayIcon);
        TrayIcon.Wnd := Self.Handle;
        TrayIcon.uID := 0;
        TrayIcon.uFlags := NIF_ICON or NIF_TIP or NIF_MESSAGE;
        TrayIcon.uCallbackMessage := MY_WM_TRAYICON;

        TrayIcon.hIcon :=  IconPic.Handle;
        TrayIcon.szTip := 'ArsClip';
        ShellAPI.Shell_notifyIcon(NIM_MODIFY, @TrayIcon);
    end;
end;


{
--========================
-- // Create/Destroy   //
--========================
}

procedure TfrmSysTrayMenu.FormCreate(Sender: TObject);
begin
    TrayPopup := TPopupMenu.Create(self);
    iconpic := TIcon.Create;

    //
    // create the initial icon
    // NIM_MODIFY will be performed for updates
    //
    TrayIcon.cbSize := SizeOf(TrayIcon);
    TrayIcon.Wnd := Self.Handle;
    TrayIcon.uID := 0;
    TrayIcon.uFlags := NIF_ICON or NIF_TIP or NIF_MESSAGE;
    TrayIcon.uCallbackMessage := MY_WM_TRAYICON;
    TrayIcon.hIcon := Application.Icon.Handle;
    TrayIcon.szTip := 'ArsClip';
    ShellAPI.Shell_notifyIcon(NIM_ADD, @TrayIcon);
end;

procedure TfrmSysTrayMenu.FormDestroy(Sender: TObject);
begin
    TrayPopup.Items.Clear;
    TrayPopup.Free;
    iconPic.free;

    //
    // Remove tray icon
    //
    ShellAPI.Shell_notifyIcon(NIM_DELETE, @TrayIcon);
end;



{
--========================
-- // Trayicon messages //
--========================

Description: On a right click on the tray icon,
    present a popup menu that allows the user to

close the program
run configuration
run permanent items
switch permanent items
flush the clipboard history

}

procedure TFrmSysTrayMenu.WMTrayIcon(var Msg: TMessage);
begin
    //
    // show Popup menu on a right click
    // show configuration form on a left double click
    // show/hide debug window on right double click
    //
    if (Msg.lparam = WM_Rbuttondown) then begin
        self.ShowPopup;
    end else if (msg.LParam = WM_LBUTTONDBLCLK ) then begin
        frmconfig.show();
    end else if (msg.LParam = WM_RBUTTONDBLCLK ) then begin
        frmMainPopup.Visible := frmMainPopup.Visible xor true;
    end;

end;


procedure TFrmSysTrayMenu.ShowPopup;
var CursorPos : TPoint;
    menu: TMenuItem;
    submenu : TMenuItem;
    i : integer;
    s : string;
    procedure ImageToBitmap(img : TImage; bmp : TBitmap);
    begin
        bmp.pixelformat := pf24bit;
        bmp.Width := img.Width;
        bmp.Height := img.Height;
        bmp.Canvas.draw(0,0, img.Picture.Graphic);
    end;
begin
    {
    |=================|
    | Configure       |
    |-----------------|
    | Permenant Items |
    | Switch to... |> |
    |-----------------|
    | Flush History   |
    | Edit History    |
    | Removed Items...|
    |-----------------|
    | About           |
    | Exit            |
    |=================|
    }
    frmMainPopup.AppendLog('WMTrayIcon right button ');

    Windows.GetCursorPos(CursorPos);
    TrayPopup.Items.Clear;
    TrayPopup.AutoHotkeys := maAutomatic;

    menu := TMenuItem.Create(TrayPopup);
    menu.caption := 'Configure';
    menu.OnClick := self.ConfigureMenuItemClickEvent;
    menu.Bitmap := FrmMainPopup.imgA.Picture.Bitmap;
    TrayPopup.Items.Add(menu);

    menu := TMenuItem.Create(TrayPopup);
    menu.caption := 'Refresh Clipboard Monitoring';
    menu.OnClick := self.RefreshMenuItemClickEvent;
    //menu.Bitmap := FrmMainPopup.imgA.Picture.Bitmap;
    TrayPopup.Items.Add(menu);


    TrayPopup.Items.Add(NewLine());

    menu := TMenuItem.Create(TrayPopup);
    menu.Caption := 'Permanent Items';
    menu.OnClick := self.SwitchMenuItemClickEvent;
    //ImageToBitmap(frmMainPopup.imgP, menu.bitmap);
    menu.Bitmap := FrmMainPopup.imgRightArrow.Picture.Bitmap;
    TrayPopup.Items.Add(menu);

    // switch to sub menu (checkmark current item)
    menu := TMenuItem.Create(TrayPopup);
    menu.Caption := 'Switch to...';
    s := frmPermanent.GetPermanentPath;
    for i := 0 to frmPermanent.PermFoldersGetCount - 1 do begin
        submenu := TMenuItem.Create(menu);
        submenu.hint := frmPermanent.PermFoldersGetItem(i);
        submenu.caption := submenu.hint;
        if (submenu.Hint = s) then begin
            submenu.Checked := true;
        end;
        submenu.OnClick := self.SwitchPermFolder;
        menu.Insert(i, submenu)
    end;
    TrayPopup.Items.Add(menu);

    TrayPopup.Items.Add(NewLine());
    menu := TMenuItem.Create(TrayPopup);
    menu.Caption := 'Flush History';
    menu.OnClick := self.FlushHistoryMenuItemClickEvent;
    TrayPopup.Items.Add(menu);

    menu := TMenuItem.Create(TrayPopup);
    menu.Caption := 'Edit History';
    menu.Bitmap := FrmMainPopup.imgEdit.Picture.Bitmap;
    menu.OnClick := self.EditHistoryMenuItemClickEvent;
    TrayPopup.Items.Add(menu);

    menu := TMenuItem.Create(TrayPopup);
    menu.Caption := 'Removed Items';
    menu.Bitmap := FrmMainPopup.imgRemoved.Picture.Bitmap;
    menu.OnClick := self.RemovedItemsMenuItemClickEvent;
    TrayPopup.Items.Add(menu);


    TrayPopup.Items.Add(NewLine());
    menu := TMenuItem.Create(TrayPopup);
    menu.caption := 'About';
    menu.OnClick := self.AboutClickEvent;
    TrayPopup.Items.Add(menu);

    menu := TMenuItem.Create(TrayPopup);
    menu.caption := 'Exit';
    menu.OnClick := self.ExitMenuItemClickEvent;
    TrayPopup.Items.Add(menu);


    frmMainPopup.AppendLog('>>tray popup');

    Windows.SetForegroundWindow(frmMainPopup.handle);
    TrayPopup.Popup(CursorPos.x, CursorPos.y);
    Windows.PostMessage(frmMainPopup.Handle, WM_NULL, 0, 0);

    frmMainPopup.AppendLog('>>tray popup done');
end;


{
  // menu item clicks
}


procedure TFrmSysTrayMenu.RefreshMenuItemClickEvent(Sender : TObject);
begin
    frmClipboardManager.RefreshClipboardMonitor;
end;
procedure TFrmSysTrayMenu.ExitMenuItemClickEvent(Sender: TObject);
begin
    // End the program nicely
    frmMainPopup.Close;
end;
procedure TFrmSysTrayMenu.SwitchMenuItemClickEvent(sender : TOBject);
begin
    frmPermanent.Show;
end;
procedure TFrmSysTrayMenu.ConfigureMenuItemClickEvent(Sender : TObject);
begin
    frmConfig.Show;
end;
procedure TFrmSysTrayMenu.SwitchPermFolder(sender : TObject);
begin
    with sender as TMenuItem do begin
        frmPermanent.SetPermanentPath( hint );
    end;
end;
procedure TFrmSysTrayMenu.FlushHistoryMenuItemClickEvent(sender : TObject);
begin
    ClipQueue.ClearQueue;
end;

procedure TFrmSysTrayMenu.EditHistoryMenuItemClickEvent(sender : TObject);
begin
    frmClipboardManager.Show;
end;

procedure TFrmSysTrayMenu.AboutClickEvent(sender : TObject);
begin
    frmAbout.show;
end;


procedure TFrmSysTrayMenu.RemovedItemsMenuItemClickEvent(sender : Tobject);
begin
    FrmRemoved.Show;
end;


procedure TfrmSysTrayMenu.ToggleTimerTimer(Sender: TObject);
begin
    self.SetIconNormal;
end;

end.
